@@ -1,4 +1,4 @@
-/* $Id: midgard.c 22049 2009-04-28 12:07:07Z piotras $
+/* $Id: midgard.c 23835 2009-11-02 08:05:38Z piotras $
  *
  * Copyright (C) 2007 Piotr Pokora <piotrek.pokora@gmail.com>
  *
@@ -28,8 +28,8 @@
 #include "midgard_object.h"
 #include "midgard_user.h"
 #include "midgard_blob.h"
-#include "midgard_sitegroup.h"
 #include "midgard_collector.h"
+#include "midgard_view.h"
 #include <libgda/libgda.h>
 
 const char *midgard_version()
@@ -50,41 +50,78 @@ void midgard_init()
 
 	type = MIDGARD_TYPE_BLOB;
 	g_assert(type != 0);
-
-	type = MIDGARD_TYPE_SITEGROUP;
-	g_assert(type != 0);
 	g_type_class_ref(type);
 
 	type = MIDGARD_TYPE_USER;
 	g_assert(type != 0);
+	g_type_class_ref(type);
 
 	type = MIDGARD_TYPE_CONNECTION;
 	g_assert(type != 0);
+	g_type_class_ref(type);
 
 	type = MIDGARD_TYPE_CONFIG;
 	g_assert(type != 0);
+	g_type_class_ref(type);
 
 	type = MIDGARD_TYPE_COLLECTOR;
 	g_assert(type != 0);
+	g_type_class_ref(type);
 
 	type = MIDGARD_TYPE_QUERY_BUILDER;
 	g_assert(type != 0);
-
-	type = MIDGARD_TYPE_REQUEST_CONFIG;
-	g_assert(type != 0);
-
+	g_type_class_ref(type);
+	
 	type = MIDGARD_TYPE_DBOBJECT;
 	g_assert(type != 0);
+	g_type_class_ref(type);
 
 	type = MIDGARD_TYPE_OBJECT;
 	g_assert(type != 0);
+	g_type_class_ref(type);
+
+	type = MIDGARD_TYPE_VIEW;
+	g_assert(type != 0);
+	g_type_class_ref(type);
 }
 
 void midgard_close(void)
 {
 	GObjectClass *klass;	
 	
-	klass = g_type_class_peek(MIDGARD_TYPE_SITEGROUP);
+	klass = g_type_class_peek(MIDGARD_TYPE_BLOB);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_USER);
+	if (klass)
+		g_type_class_unref(klass);
+	
+	klass = g_type_class_peek(MIDGARD_TYPE_CONNECTION);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_CONFIG);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_COLLECTOR);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_QUERY_BUILDER);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_DBOBJECT);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_OBJECT);
+	if (klass)
+		g_type_class_unref(klass);
+
+	klass = g_type_class_peek(MIDGARD_TYPE_VIEW);
 	if (klass)
 		g_type_class_unref(klass);
 }
