@@ -36,10 +36,13 @@
 #include "midgard_core_config.h"
 #include "midgard_connection.h"
 #include "midgard_error.h"
+#include "midgard_core_object_class.h"
 
-#define AUTH_NORMAL_STR "Normal"
-#define AUTH_PAM_STR "PAM"
-#define AUTH_TRUSTED_STR "Trusted"
+#define AUTH_NORMAL_STR		"Normal"
+#define AUTH_PAM_STR		"PAM"
+#define AUTH_TRUSTED_STR	"Trusted"
+
+static gchar *__get_default_confdir(void);
 
 /* Properties */
 enum {
@@ -49,7 +52,6 @@ enum {
 	MIDGARD_CONFIG_DBPASS, 
 	MIDGARD_CONFIG_HOST,
 	MIDGARD_CONFIG_LOGFILENAME,
-	MIDGARD_CONFIG_DEFAULT_LANG,
 	MIDGARD_CONFIG_LOGLEVEL,
 	MIDGARD_CONFIG_TABLECREATE,
 	MIDGARD_CONFIG_TABLEUPDATE,
@@ -87,75 +89,10 @@ MidgardConfig *midgard_config_new(void)
 {
 	MidgardConfig *self = 
 		g_object_new(MIDGARD_TYPE_CONFIG, NULL);
+
 	return self;
 }
 
-/* Get configuration directory path.
- * We define this directory according to MIDGARD_LIB_PREFIX 
- * We also cache the result as it is costs a lot to do g_dir_open() each time we calculate paths
- * This means that returned mdirs* should be constant.
- */
- 
-const mdirs *midgard_core_config_get_config_dirs()
-{
-	static mdirs *iconf = NULL;
-	/* GDir *dir; */
-	
-	if (iconf != NULL) return iconf;
-	
-	iconf = g_new(mdirs,1);
-	
-	if (g_str_equal(MIDGARD_LIB_PREFIX, "/usr")) {
-
-		/* Set typical FSH directory */
-		iconf->confdir = g_strconcat("/etc/", MIDGARD_PACKAGE_NAME, "/conf.d", NULL);
-		iconf->blobdir = g_strconcat("/var/lib/", MIDGARD_PACKAGE_NAME, "/blobs", NULL);
-		iconf->cachedir = g_strconcat("/var/cache/", MIDGARD_PACKAGE_NAME, NULL);
-		iconf->vardir = g_strconcat("/var/lib/", MIDGARD_PACKAGE_NAME, NULL);
-
-	} else {
-
-		/* Set any directory which follows midgard prefix */
-		iconf->confdir = g_strconcat(MIDGARD_LIB_PREFIX, "/etc/", 
-				MIDGARD_PACKAGE_NAME, "/conf.d", NULL);
-		iconf->blobdir = g_strconcat(MIDGARD_LIB_PREFIX, "/var/lib/", 
-				MIDGARD_PACKAGE_NAME, "/blobs", NULL);
-		iconf->cachedir = g_strconcat(MIDGARD_LIB_PREFIX, "/var/cache/", 
-				MIDGARD_PACKAGE_NAME, NULL);
-		iconf->vardir = g_strconcat(MIDGARD_LIB_PREFIX, "/var/lib/", 
-				MIDGARD_PACKAGE_NAME, NULL);
-	}
-
-	/* Set blobdir to /var/local when /usr/local was set as prefix */
-	if (g_str_equal(MIDGARD_LIB_PREFIX, "/usr/local")) {
-
-		iconf->blobdir = g_strconcat("/var/local/lib/", MIDGARD_PACKAGE_NAME, 
-				"/blobs", NULL);
-		iconf->cachedir = g_strconcat("/var/local/cache/", MIDGARD_PACKAGE_NAME, NULL);
-		iconf->vardir = g_strconcat("/var/local/lib/", MIDGARD_PACKAGE_NAME, NULL);
-
-	}
-	
-	iconf->sharedir = g_strconcat( MIDGARD_LIB_PREFIX, "/share/", 
-			MIDGARD_PACKAGE_NAME, NULL);
-
-	return iconf;
-}
-
-void midgard_core_config_dirs_free(mdirs **iconf)
-{
-	if (!(iconf && *iconf)) return;
-
-	g_free((*iconf)->confdir);
-	g_free((*iconf)->blobdir);
-	g_free((*iconf)->sharedir);
-	g_free((*iconf)->vardir);
-	g_free((*iconf)->cachedir);
-
-	g_free(*iconf);
-	*iconf = NULL;
-}
-
 void __set_dbtype(MidgardConfig *self, const gchar *tmpstr)
 {
 	const gchar *_dbtype = tmpstr;
@@ -180,6 +117,7 @@ void __set_dbtype(MidgardConfig *self, const gchar *tmpstr)
 		_dbtype = "MySQL";
 	}
 
+	g_free (self->dbtype);
 	self->dbtype = g_strdup(_dbtype);
 }
 
@@ -202,7 +140,7 @@ void __create_log_dir(const gchar *path)
 	}
 }
 
-static gboolean __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile, const gchar *filename, const mdirs *iconf)
+static void __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile, const gchar *filename)
 {
 	g_assert(self != NULL);
 	g_assert(keyfile != NULL);
@@ -255,14 +193,6 @@ static gboolean __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile
 	}
 	g_free(tmpstr);
 
-	/* Get default language */
-	tmpstr = g_key_file_get_string(keyfile, "MidgardDatabase", "DefaultLanguage", NULL); 
-	if(tmpstr != NULL && *tmpstr != '\0') {
-		g_free(self->default_lang);
-		self->default_lang = g_strdup(tmpstr);
-	}
-	g_free(tmpstr);
-
 	/* Get log filename */
 	tmpstr = g_key_file_get_string(keyfile, "MidgardDatabase", "Logfile", NULL);
 	GIOChannel *channel = NULL;
@@ -362,63 +292,51 @@ static gboolean __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile
 	
 	} else {
 
+		gchar *db_blobdir = g_build_path(G_DIR_SEPARATOR_S, 
+				self->blobdir, self->database, NULL);
 		g_free(self->blobdir);
-		self->blobdir = 
-			g_strconcat(iconf->blobdir, "/", self->database, NULL);
-
+		self->blobdir = db_blobdir;
 	}
 
 	/* ShareDir */
 	tmpstr = g_key_file_get_string(keyfile, "MidgardDir", "ShareDir", NULL);
-	if(tmpstr == NULL || *tmpstr == '\0') {
+	if(tmpstr != NULL && *tmpstr != '\0') {
 
 		g_free(self->sharedir);
-		self->sharedir = g_strdup(iconf->sharedir);
-
-	} else {
-
-		g_free(self->sharedir);
-		self->sharedir = g_strdup(tmpstr);
-		g_free(tmpstr);
+		self->sharedir = tmpstr;
 	}
 
 	/* VarDir */
 	tmpstr = g_key_file_get_string(keyfile, "MidgardDir", "VarDir", NULL);
-	if(tmpstr == NULL || *tmpstr == '\0') {
+	if(tmpstr != NULL || (tmpstr && *tmpstr != '\0')) {
 
 		g_free(self->vardir);
-		self->vardir = g_strdup(iconf->vardir);
-
-	} else {
-
-		g_free(self->vardir);
-		self->vardir = g_strdup(tmpstr);
-		g_free(tmpstr);
+		self->vardir = tmpstr;
 	}
 
 	/* CacheDir */
 	tmpstr = g_key_file_get_string(keyfile, "MidgardDir", "ShareDir", NULL);
-	if(tmpstr == NULL || *tmpstr == '\0') {
+	if(tmpstr != NULL || (tmpstr && *tmpstr == '\0')) {
 
 		g_free(self->cachedir);
-		self->cachedir = g_strdup(iconf->cachedir);
-
-	} else {
-
-		g_free(self->cachedir);
-		self->cachedir = g_strdup(tmpstr);
-		g_free(tmpstr);
-	}
+		self->cachedir = tmpstr;
+	} 
 
 	/* SchemaDir */
 	g_free(self->schemadir);
-	self->schemadir = g_build_path(G_DIR_SEPARATOR_S, self->sharedir, "schema", NULL);
+	self->schemadir = g_build_path(G_DIR_SEPARATOR_S, self->sharedir, MIDGARD_CONFIG_RW_SCHEMA, NULL);
+
+	/* ViewsDir */
+	g_free(self->viewsdir);
+	self->viewsdir = g_build_path(G_DIR_SEPARATOR_S, self->sharedir, MIDGARD_CONFIG_RW_VIEW, NULL);
 
 	/* We will free it when config is unref */
 	self->priv->keyfile = keyfile;
 
 	if (filename)
 		self->priv->configname = g_strdup(filename);
+
+	return;
 }
 
 /**
@@ -440,25 +358,21 @@ static gboolean __set_config_from_keyfile(MidgardConfig *self, GKeyFile *keyfile
  */ 
 gboolean midgard_config_read_file(MidgardConfig *self, const gchar *filename, gboolean user, GError **error)
 {
-	const mdirs *iconf;
 	gchar *fname = NULL;	
 	GKeyFile *keyfile;	
 
-
 	g_assert(self != NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
 	if(filename == NULL || *filename == '\0') {
 		
-		*error = g_error_new(MGD_GENERIC_ERROR, 
-				MGD_ERR_INVALID_NAME, 
-				"Configuration name can not be empty");
-		return FALSE;
+			g_set_error(error, MGD_GENERIC_ERROR, 
+					MGD_ERR_INVALID_NAME, 
+					"Configuration name can not be empty");
+			
+			return FALSE;
 	}
 	
-	if ((iconf = midgard_core_config_get_config_dirs()) == NULL)
-		return FALSE;
-
 	if(user) {
 		gchar *_umcd = g_strconcat(".", MIDGARD_PACKAGE_NAME, NULL);
 		fname = g_build_path (G_DIR_SEPARATOR_S, 
@@ -466,16 +380,27 @@ gboolean midgard_config_read_file(MidgardConfig *self, const gchar *filename, gb
 		g_free(_umcd);
 	} else {
 		fname = g_build_path (G_DIR_SEPARATOR_S,
-				iconf->confdir, "/", filename, NULL);
+				self->confdir, "/", filename, NULL);
 	}
 
 	GError *kf_error = NULL;
 	keyfile = g_key_file_new();
+
 	if (!g_key_file_load_from_file(keyfile, fname, G_KEY_FILE_NONE, &kf_error)) {
-		*error = g_error_new(MGD_GENERIC_ERROR,
-				MGD_ERR_INTERNAL,
-				"Can not open %s. %s" , fname, kf_error->message);
-		g_clear_error(&kf_error);
+
+		if (kf_error && kf_error->message) {
+
+			g_propagate_error (error, kf_error);
+		
+		} else { 
+
+			g_set_error (error, MGD_GENERIC_ERROR,
+					MGD_ERR_INTERNAL,
+					"Can not open %s. %s" , fname, kf_error && kf_error->message ? kf_error->message : "Unknown reason.");
+
+			g_clear_error(&kf_error);
+		}
+
 		g_free(fname);
 		return FALSE;
 	}
@@ -499,7 +424,7 @@ gboolean midgard_config_read_file(MidgardConfig *self, const gchar *filename, gb
 		}
 	}
 
-	__set_config_from_keyfile(self, keyfile, filename, iconf);
+	__set_config_from_keyfile(self, keyfile, filename);
 
 	return TRUE;
 }
@@ -519,31 +444,25 @@ gboolean midgard_config_read_file_at_path(MidgardConfig *self, const gchar *file
 
 	GKeyFile *keyfile = NULL;
 
-	if(filepath == NULL || *filepath == '\0') {
+	if (filepath == NULL || *filepath == '\0') {
 		
-		*error = g_error_new(MGD_GENERIC_ERROR, 
-				MGD_ERR_INVALID_NAME, 
-				"Configuration file path can not be empty");
+			g_set_error (error, MGD_GENERIC_ERROR, MGD_ERR_INVALID_NAME, "Configuration file path can not be empty");
+	
 		return FALSE;
 	}
 
 	GError *kf_error = NULL;
 	keyfile = g_key_file_new();
-	if (!g_key_file_load_from_file (
-				keyfile, filepath, G_KEY_FILE_NONE, &kf_error)) {
-		
-		*error = g_error_new(MGD_GENERIC_ERROR,
-				MGD_ERR_INTERNAL,
-				"Can not open %s. %s" , filepath, kf_error->message);
+
+	if (!g_key_file_load_from_file(keyfile, filepath, G_KEY_FILE_NONE, &kf_error)) {
+
+		g_set_error (error, MGD_GENERIC_ERROR, MGD_ERR_INTERNAL, "Can not open %s. %s" , filepath, kf_error->message);
+
 		g_clear_error(&kf_error);
 		return FALSE;
 	}
 
-	const mdirs *iconf = midgard_core_config_get_config_dirs();
-	if (iconf == NULL)
-		return FALSE;
-
-	__set_config_from_keyfile(self, keyfile, filepath, iconf);
+	__set_config_from_keyfile(self, keyfile, filepath);
 
 	return TRUE;
 }
@@ -568,18 +487,14 @@ gboolean midgard_config_read_data(MidgardConfig *self, const gchar *data, GError
 	if (!g_key_file_load_from_data (
 				keyfile, data, strlen(data), G_KEY_FILE_NONE, &kf_error)) {
 		
-		*error = g_error_new(MGD_GENERIC_ERROR,
+		g_set_error (error, MGD_GENERIC_ERROR,
 				MGD_ERR_INTERNAL,
 				"Can not read the configuration data");
 		g_clear_error(&kf_error);
 		return FALSE;
 	}
 
-	const mdirs *iconf = midgard_core_config_get_config_dirs();
-	if (iconf == NULL)
-		return FALSE;
-
-	__set_config_from_keyfile(self, keyfile, NULL, iconf);
+	__set_config_from_keyfile(self, keyfile, NULL);
 
 	return TRUE;
 }
@@ -600,7 +515,6 @@ gboolean midgard_config_read_data(MidgardConfig *self, const gchar *data, GError
 gchar **midgard_config_list_files(gboolean user)
 {
 	gchar *config_dir;
-	const mdirs *iconf;
 
 	if(user) {
 		
@@ -616,12 +530,8 @@ gchar **midgard_config_list_files(gboolean user)
 		} 
 
 	} else {
-
-		iconf = midgard_core_config_get_config_dirs();
-		if(!iconf)
-			return NULL;
 		
-		config_dir = g_strdup(iconf->confdir);
+		config_dir = __get_default_confdir();
 	}
 
 	GError *error = NULL;
@@ -665,12 +575,13 @@ gchar **midgard_config_list_files(gboolean user)
  * @self: #MidgardConfig instance
  * @name: configuration filename
  * @user: system or home directory switch
+ * @error: pointer to store GError
  *
  * Saves configuration file for the given #MidgardConfig.
  *
  * This method saves configuration file with the given name.
  * If third user parameter is set to %TRUE, then configuration file will 
- * be saved in ~/.midgard/conf.d directory.
+ * be saved in ~/.midgard2/conf.d directory.
  *
  * User's conf.d directory will be created if doesn't exist.
  * 
@@ -751,21 +662,15 @@ gboolean midgard_config_save_file(MidgardConfig *self,
 
 	} else {
 		
-		const mdirs *iconf = midgard_core_config_get_config_dirs();
-		if(!iconf)
-			return FALSE;
-
-		if(!g_file_test((const gchar *)iconf->confdir,
+		if(!g_file_test((const gchar *)self->confdir,
 					G_FILE_TEST_EXISTS)
-				|| (!g_file_test((const gchar *)iconf->confdir,
+				|| (!g_file_test((const gchar *)self->confdir,
 						G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
-			g_warning("%s doesn't exist or is not a directory!", 
-					iconf->confdir);
+			g_warning("%s doesn't exist or is not a directory!", self->confdir);
 			return FALSE;
 		}
 
-		cnfpath = g_build_path (G_DIR_SEPARATOR_S,
-				iconf->confdir, name, NULL);
+		cnfpath = g_build_path (G_DIR_SEPARATOR_S, self->confdir, name, NULL);
 
 	}
 
@@ -827,24 +732,28 @@ gboolean midgard_config_save_file(MidgardConfig *self,
 	gsize length;
 	GError *kf_error = NULL;
 	
-	gchar *content = g_key_file_to_data(self->priv->keyfile,
-			&length, &kf_error);
+	gchar *content = g_key_file_to_data (self->priv->keyfile, &length, &kf_error);
 
-	if(length < 1) {
-		if(kf_error)
-			g_propagate_error(error, kf_error);	
+	if (length < 1) {
+	
+		if (kf_error)
+			g_propagate_error (error, kf_error);	
+	
 		return FALSE;
 	}
 
-	if(kf_error)
-		g_clear_error(&kf_error);
+	if (kf_error)
+		g_clear_error (&kf_error);
 
-	gboolean saved = g_file_set_contents(cnfpath, 
-			content, length, &kf_error);
+	gboolean saved = g_file_set_contents(cnfpath, content, length, &kf_error);
 
-	if(!saved) {
-		if(kf_error)
-			g_propagate_error(error, kf_error);
+	g_free (content);
+
+	if (!saved) {
+
+		if (kf_error)
+			g_propagate_error (error, kf_error);
+	
 		return FALSE;
 	}
 
@@ -862,97 +771,14 @@ gboolean midgard_config_save_file(MidgardConfig *self,
  * @self: #MidgardConfig instance
  * @mgd: #MidgardConnection handler
  *
- * Creates basic Midgard database tables.
- *
- * Tables created by this method: repligard, sitegroup, midgard_user
- *
+ * Deprecated:9.09: Replaced with #MidgardStorage
+ * 
  * Returns: %TRUE if tables has been created , %FALSE otherwise
  */ 
 gboolean midgard_config_create_midgard_tables(MidgardConfig *self,
 		MidgardConnection *mgd)
 {
-	if(mgd == NULL) {
-		g_warning("Connection should be established for midgard_config_create_midgard_tables");
-		return FALSE;
-	}
-
-	gboolean ret =  midgard_core_query_create_basic_db(mgd);
-
-	if(!ret)
-		return FALSE;
-	
-	/* TODO, Try to SELECT before INSERT is done */
-	
-	const gchar *sql = "SELECT guid FROM midgard_user WHERE login='root' AND sitegroup=0";
-	GdaDataModel *model = midgard_core_query_get_model(mgd, sql);
-	
-	if(model && gda_data_model_get_n_rows(model) == 0) {
-		/* This is initial db so we have to "import" SG0 default Admin */
-		/* midgard_user record */
-		sql = "INSERT INTO midgard_user (guid, sitegroup, active, hashed, "
-			"hashtype, user_type, login) VALUES ('f6b665f1984503790ed91f39b11b5392', "
-			"0, 1, 'password', 1, 2, 'root');";
-		
-		midgard_core_query_execute(mgd, sql, TRUE);
-	}
-
-	if(model)
-		g_object_unref(model);
-
-	/* quota table */
-	MidgardObjectClass *quota_klass = 
-		MIDGARD_OBJECT_GET_CLASS_BY_NAME("midgard_quota");
-	midgard_config_create_class_table(self, quota_klass, mgd);
-
-	/* person table */
-	MidgardObjectClass *person_klass = 
-		MIDGARD_OBJECT_GET_CLASS_BY_NAME("midgard_person");
-	midgard_config_create_class_table(self, person_klass, mgd);
-
-	/* member table */
-	MidgardObjectClass *member_klass = 
-		MIDGARD_OBJECT_GET_CLASS_BY_NAME("midgard_member");
-	midgard_config_create_class_table(self, member_klass, mgd);
-
-	sql = "SELECT lastname, firstname FROM person WHERE sitegroup=0 AND guid='f6b665f1984503790ed91f39b11b5392'";
-
-	model = midgard_core_query_get_model(mgd, sql);
-
-	guint ret_rows = 0;
-	if(model)
-		ret_rows = gda_data_model_get_n_rows(model);
-
-	if(!model || ret_rows == 0) {
-		
-		/* midgard_person record */
-		sql = "INSERT INTO person (guid, sitegroup, birthdate, homepage, email, extra, "
-			"metadata_creator, lastname, firstname, metadata_created, metadata_revision, username, password ) "
-			"VALUES ('f6b665f1984503790ed91f39b11b5392', 0, '1999-05-18 14:40:01', "
-			"'http://www.midgard-project.org', 'dev@midgard-project.org', "
-			"'Default administrator account for Midgard', 'f6b665f1984503790ed91f39b11b5392', "
-			"'Administrator', 'Midgard', '1999-05-18 14:40:01', 1, 'root', '**password')";
-		
-		midgard_core_query_execute(mgd, sql, TRUE);
-
-		/* Create root's repligard entry */
-		sql = "INSERT INTO repligard (guid, typename, sitegroup, object_action) "
-			"VALUES ('f6b665f1984503790ed91f39b11b5392', 'midgard_person', 0, 0)";
-		midgard_core_query_execute(mgd, sql, TRUE);
-		
-		/* midgard root's member record */
-		sql = "INSERT INTO member (guid, uid, gid) VALUES ('df351bde7c09bce7ad2cf14f2eaf7fb7', 1, 0)";
-		midgard_core_query_execute(mgd, sql, TRUE);
-
-		/* Create root's membership repligard entry */
-		sql = "INSERT INTO repligard (guid, typename, sitegroup, object_action) "
-			"VALUES ('df351bde7c09bce7ad2cf14f2eaf7fb7', 'midgard_member', 0, 0)";
-		midgard_core_query_execute(mgd, sql, TRUE);
-	}
-
-	if(model)
-		g_object_unref(model);
-
-	return TRUE;
+	return FALSE;
 }
 
 /**
@@ -976,6 +802,8 @@ gboolean midgard_config_create_midgard_tables(MidgardConfig *self,
  * and it's integer ( or unsigned one ) type. In such case , field is also 
  * a primary field. @self parameter may be NULL if the purpose is to call static method.
  *
+ * Deprecated:9.09: Replaced with #MidgardStorage 
+ *
  * Returns: %TRUE if table has been created, %FALSE otherwise.
  */
 gboolean midgard_config_create_class_table(MidgardConfig *self,
@@ -985,10 +813,10 @@ gboolean midgard_config_create_class_table(MidgardConfig *self,
 		g_warning("Connection should be established for midgard_config_create_class_table");
 		return FALSE;
 	}
-	
+
 	g_return_val_if_fail(klass != NULL, FALSE);
 
-	return midgard_core_query_create_class_storage(mgd, klass);
+	return midgard_core_query_create_class_storage(mgd, MIDGARD_DBOBJECT_CLASS(klass));
 }
 
 /**
@@ -1001,6 +829,8 @@ gboolean midgard_config_create_class_table(MidgardConfig *self,
  *
  * Creates columns if registered for class, but do not exist in table.
  *
+ * Deprecated:9.09: Replaced with #MidgardStorage 
+ *
  * Returns: %TRUE if table has been updated, %FALSE otherwise.
  */
 gboolean midgard_config_update_class_table(MidgardConfig *self,
@@ -1010,10 +840,36 @@ gboolean midgard_config_update_class_table(MidgardConfig *self,
 		g_warning("Connection should be established for midgard_config_update_class_table");
 		return FALSE;
 	}
-	
+
 	g_return_val_if_fail(klass != NULL, FALSE);
 
-	return midgard_core_query_update_class_storage(mgd, klass);
+	return midgard_core_query_update_class_storage(mgd, MIDGARD_DBOBJECT_CLASS(klass));
+}
+
+/** 
+ * midgard_config_class_table_exists:
+ * @self: #MidgardConfig instance or NULL
+ * @klass: a pointer to #MidgardObjectClass
+ * @mgd: #MidgardConnection pointer
+ * 
+ * Checks if database table exists for given class.
+ *
+ * Deprecated:9.09: Replaced with #MidgardStorage 
+ *
+ * Returns: %TRUE if table exists, %FALSE otherwise
+ */
+gboolean midgard_config_class_table_exists(MidgardConfig *self,
+		MidgardObjectClass *klass, MidgardConnection *mgd)
+{
+	g_assert(klass != NULL);
+	g_assert(mgd != NULL);
+
+	const gchar *tablename = 
+		midgard_core_class_get_table(MIDGARD_DBOBJECT_CLASS(klass));
+
+	g_return_val_if_fail(tablename != NULL, FALSE);
+
+	return midgard_core_table_exists(mgd, tablename);	
 }
 
 gboolean __create_base_blobdir(const gchar *path)
@@ -1082,7 +938,7 @@ gboolean midgard_config_create_blobdir(MidgardConfig *self)
 {
 	g_assert(self != NULL);
 
-	gboolean err = FALSE;
+	gboolean err = FALSE;	
 	
 	if (self->blobdir == NULL || *self->blobdir == '\0')
 	{
@@ -1110,88 +966,197 @@ gboolean midgard_config_create_blobdir(MidgardConfig *self)
 	return TRUE;
 }
 
+static gchar *
+__get_default_confdir (void)
+{
+	gchar *confdir = NULL;
+	confdir = g_strdup (getenv("MIDGARD_ENV_GLOBAL_CONFDIR"));
+
+	if (!confdir || (confdir && g_str_equal (confdir, ""))) {
+		g_free(confdir);
+
+		if (g_str_equal(MIDGARD_LIB_PREFIX, "/usr")) {
+			/* Using the standard prefix "/usr", so, choosing "/etc" */
+			confdir = g_build_path(G_DIR_SEPARATOR_S, " ", "etc", MIDGARD_PACKAGE_NAME, "conf.d", NULL);
+		} else {
+			/* Using some other prefix, so choosing "prefix/etc" */
+			confdir = g_build_path(G_DIR_SEPARATOR_S, MIDGARD_LIB_PREFIX, "etc", MIDGARD_PACKAGE_NAME, "conf.d", NULL);
+		}
+	}
+
+	return g_strchug(confdir);
+}
+
 static void __config_struct_new(MidgardConfig *self)
 {
 	self->dbtype = g_strdup("MySQL");
-	self->mgdusername = g_strdup("root");
+	self->mgdusername = g_strdup("admin");
 	self->mgdpassword = g_strdup("password");
 	self->host = g_strdup("localhost");
 	self->database = g_strdup("midgard");	
 	self->dbuser = g_strdup("midgard");
 	self->dbpass = g_strdup("midgard");
-	self->logfilename = NULL;
-	self->default_lang = NULL;
+	self->logfilename = g_strdup ("");;
 	self->loglevel = g_strdup("warn");
-	self->pamfile = NULL;
+	self->pamfile = g_strdup ("");
+
+	self->tablecreate = FALSE;
+	self->tableupdate = FALSE;
+	self->testunit = FALSE;
 
 	/* MidgardDir */
-	self->blobdir = NULL;
-	self->sharedir = NULL;
-	self->vardir = NULL;
-	self->cachedir = NULL;
-	self->schemadir = NULL;
-	self->schemadir = g_strdup(MIDGARD_LSCHEMA_DIR);
-}
+	/* Set directories using particular order:
+	   1. Get environment variables
+	   2. If above are not set, try to determine FSH defaults */
+	self->blobdir = g_strdup(getenv("MIDGARD_ENV_GLOBAL_BLOBDIR"));
+	self->sharedir = g_strdup(getenv("MIDGARD_ENV_GLOBAL_SHAREDIR"));
+	self->vardir = g_strdup(getenv("MIDGARD_ENV_GLOBAL_VARDIR"));
+	self->cachedir = g_strdup(getenv("MIDGARD_ENV_GLOBAL_CACHEDIR"));
 
-void __midgard_config_struct_free(MidgardConfig *self, gboolean object)
-{
-	if(self->mgdusername)
-		g_free(self->mgdusername);
-	if(self->mgdpassword)
-		g_free(self->mgdpassword);
-	if(self->host)
-		g_free(self->host);
-	if(self->database)
-		g_free(self->database);
-	if(self->dbuser)
-		g_free(self->dbuser);
-	if(self->dbpass)
-		g_free(self->dbpass);
-	if(self->logfilename)
-		g_free(self->logfilename);
-	if(self->schemadir)
-		g_free(self->schemadir);
-	if(self->default_lang)
-		g_free(self->default_lang);
-	if(self->loglevel)
-		g_free(self->loglevel);
-	if(self->pamfile)
-		g_free(self->pamfile);
-
-	if(self->priv->keyfile)
-		g_key_file_free(self->priv->keyfile);
-	self->priv->keyfile = NULL;
-
-	g_free(self->priv);
-	self->priv = NULL;
-
-	if(!object) {
-		g_free(self);
-		self = NULL;
+	if (!self->sharedir 
+			|| (self->sharedir && g_str_equal(self->sharedir, ""))) {
+		g_free(self->sharedir);
+		self->sharedir = g_build_path(G_DIR_SEPARATOR_S, 
+				MIDGARD_LIB_PREFIX, "share", MIDGARD_PACKAGE_NAME, NULL);
 	}
 
-	/* MidgardDir */
-	if(self->blobdir)
-		g_free(self->blobdir);
-	self->blobdir = NULL;
+	if (g_str_equal(MIDGARD_LIB_PREFIX, "/usr")) {
 
-	if(self->sharedir)
-		g_free(self->sharedir);
-	self->sharedir = NULL;
+		if (!self->blobdir 
+				|| (self->blobdir && g_str_equal(self->blobdir, ""))) {
+			g_free(self->blobdir);
+			self->blobdir = g_build_path(G_DIR_SEPARATOR_S, 
+					G_DIR_SEPARATOR_S, "var", "lib", MIDGARD_PACKAGE_NAME, "blobs", NULL);
+		}
 
-	if(self->cachedir)
-		g_free(self->cachedir);
-	self->cachedir = NULL;
+		if (!self->vardir 
+				|| (self->vardir && g_str_equal(self->vardir, ""))) {
+			g_free(self->vardir);
+			self->vardir = g_build_path(G_DIR_SEPARATOR_S, 
+					G_DIR_SEPARATOR_S, "var", "lib", MIDGARD_PACKAGE_NAME, NULL);
+		}
 
-	if(self->vardir)
-		g_free(self->vardir);
-	self->vardir = NULL;
+		if (!self->cachedir 
+				|| (self->cachedir && g_str_equal(self->cachedir, ""))) {
+			g_free(self->cachedir);
+			self->cachedir = g_build_path(G_DIR_SEPARATOR_S, 
+					G_DIR_SEPARATOR_S, "var", "cache", MIDGARD_PACKAGE_NAME, NULL);
+		}
+
+	} else if (g_str_equal(MIDGARD_LIB_PREFIX, "/usr/local")) {
+
+		if (!self->blobdir 
+				|| (self->blobdir && g_str_equal(self->blobdir, ""))) {
+			g_free(self->blobdir);
+			self->blobdir = g_build_path(G_DIR_SEPARATOR_S, 
+					G_DIR_SEPARATOR_S, "var", "local", "lib", MIDGARD_PACKAGE_NAME, "blobs", NULL);
+		}
+
+		if (!self->vardir 
+				|| (self->vardir && g_str_equal(self->vardir, ""))) {
+			g_free(self->vardir);
+			self->vardir = g_build_path(G_DIR_SEPARATOR_S, 
+					G_DIR_SEPARATOR_S, "var", "local", "lib", MIDGARD_PACKAGE_NAME, NULL);
+		}
+
+		if (!self->cachedir 
+				|| (self->cachedir && g_str_equal(self->cachedir, ""))) {
+			g_free(self->cachedir);
+			self->cachedir = g_build_path(G_DIR_SEPARATOR_S, 
+					G_DIR_SEPARATOR_S, "var", "local", "cache", MIDGARD_PACKAGE_NAME, NULL);
+		}
+	
+	} else {
+
+		if (!self->blobdir 
+				|| (self->blobdir && g_str_equal(self->blobdir, ""))) {
+			g_free(self->blobdir);
+			self->blobdir = g_build_path(G_DIR_SEPARATOR_S, 
+					MIDGARD_LIB_PREFIX, "var", "lib", MIDGARD_PACKAGE_NAME, "blobs", NULL);
+		}
+
+		if (!self->vardir 
+				|| (self->vardir && g_str_equal(self->vardir, ""))) {
+			g_free(self->vardir);
+			self->vardir = g_build_path(G_DIR_SEPARATOR_S, 
+					MIDGARD_LIB_PREFIX, "var", "lib", MIDGARD_PACKAGE_NAME, NULL);
+		}
+
+		if (!self->cachedir 
+				|| (self->cachedir && g_str_equal(self->cachedir, ""))) {
+			g_free(self->cachedir);
+			self->cachedir = g_build_path(G_DIR_SEPARATOR_S, 
+					MIDGARD_LIB_PREFIX, "var", "cache", MIDGARD_PACKAGE_NAME, NULL);
+		}
+	}
+
+	self->confdir = __get_default_confdir();
 }
 
-void midgard_config_struct_free(MidgardConfig *config)
+void 
+__midgard_config_struct_free (MidgardConfig *self)
 {
-	g_assert(config != NULL);	
-	__midgard_config_struct_free(config, FALSE);
+	g_free (self->mgdusername);
+	self->mgdusername = NULL;
+
+	g_free (self->mgdpassword);
+	self->mgdpassword = NULL;
+	
+	g_free (self->host);
+	self->host = NULL;
+	
+	g_free (self->database);	
+	self->database = NULL;
+
+	g_free (self->dbuser);
+	self->dbuser = NULL;
+
+	g_free (self->dbpass);
+	self->dbpass = NULL;
+
+	g_free (self->logfilename);
+	self->logfilename = NULL;
+
+	g_free (self->schemadir);
+	self->schemadir = NULL;
+		
+	g_free (self->loglevel);
+	self->loglevel = NULL;
+	
+	g_free (self->pamfile);
+	self->pamfile = NULL;
+	
+	if (self->priv->keyfile)
+		g_key_file_free (self->priv->keyfile);
+	self->priv->keyfile = NULL;
+
+	g_free (self->priv);
+	self->priv = NULL;
+
+	g_free (self->confdir);
+	self->confdir = NULL;
+
+	/* MidgardDir */
+	if (self->blobdir)
+		g_free (self->blobdir);
+	self->blobdir = NULL;
+
+	if (self->sharedir)
+		g_free (self->sharedir);
+	self->sharedir = NULL;
+
+	if (self->cachedir)
+		g_free (self->cachedir);
+	self->cachedir = NULL;
+
+	if (self->vardir)
+		g_free (self->vardir);
+	self->vardir = NULL;
+
+	if (self->dbtype)
+		g_free (self->dbtype);
+	self->dbtype = NULL;
+
 }
 
 /* API helpers, getters */
@@ -1243,12 +1208,6 @@ const gchar *midgard_config_get_logfile(MidgardConfig *self)
 	return self->logfilename;
 }
 
-const gchar *midgard_config_get_default_language(MidgardConfig *self)
-{
-	g_assert(self != NULL);	
-	return self->default_lang;
-}
-
 const gchar *midgard_config_get_loglevel(MidgardConfig *self)
 {
 	g_assert(self != NULL);	
@@ -1303,6 +1262,8 @@ const gchar *midgard_config_get_authtype(MidgardConfig *self)
 			return AUTH_TRUSTED_STR;
 			break;
 	}
+
+	return NULL;
 }
 
 const gchar *midgard_config_get_pamfile(MidgardConfig *self)
@@ -1314,7 +1275,7 @@ const gchar *midgard_config_get_pamfile(MidgardConfig *self)
 /* setters */
 void midgard_config_set_database_name(MidgardConfig *self, const gchar *name)
 {
-	g_assert(self != NULL || name != NULL);
+	g_assert(self != NULL && name != NULL);
 	
 	if(self->database)
 		g_free(self->database);
@@ -1324,17 +1285,19 @@ void midgard_config_set_database_name(MidgardConfig *self, const gchar *name)
 
 void midgard_config_set_database_type(MidgardConfig *self, const gchar *type)
 {
-	g_assert(self != NULL || type != NULL);
+	g_assert(self != NULL && type != NULL);
 	
 	if(self->dbtype)
 		g_free(self->dbtype);
 
+	self->dbtype = NULL;
+
 	__set_dbtype(self, type);
 }
 
 void midgard_config_set_database_username(MidgardConfig *self, const gchar *username)
 {
-	g_assert(self != NULL || username != NULL);
+	g_assert(self != NULL && username != NULL);
 
 	if(self->dbuser)
 		g_free(self->dbuser);
@@ -1344,7 +1307,7 @@ void midgard_config_set_database_username(MidgardConfig *self, const gchar *user
 
 void midgard_config_set_database_password(MidgardConfig *self, const gchar *password)
 {
-	g_assert(self != NULL || password != NULL);
+	g_assert(self != NULL && password != NULL);
 
 	if(self->dbpass)
 		g_free(self->dbpass);
@@ -1354,7 +1317,7 @@ void midgard_config_set_database_password(MidgardConfig *self, const gchar *pass
 
 void midgard_config_set_database_host(MidgardConfig *self, const gchar *host)
 {
-	g_assert(self != NULL || host != NULL);
+	g_assert(self != NULL && host != NULL);
 
 	if(self->host)
 		g_free(self->host);
@@ -1370,7 +1333,7 @@ void midgard_config_set_database_port(MidgardConfig *self, guint port)
 
 void midgard_config_set_blobdir(MidgardConfig *self, const gchar *blobdir)
 {
-	g_assert(self != NULL || blobdir != NULL);
+	g_assert(self != NULL && blobdir != NULL);
 
 	if(self->blobdir)
 		g_free(self->blobdir);
@@ -1380,7 +1343,7 @@ void midgard_config_set_blobdir(MidgardConfig *self, const gchar *blobdir)
 
 void midgard_config_set_logfile(MidgardConfig *self, const gchar *logfile)
 {
-	g_assert(self != NULL || logfile != NULL);
+	g_assert(self != NULL && logfile != NULL);
 
 	if(self->logfilename)
 		g_free(self->logfilename);
@@ -1388,19 +1351,9 @@ void midgard_config_set_logfile(MidgardConfig *self, const gchar *logfile)
 	self->logfilename = g_strdup(logfile);
 }
 
-void midgard_config_set_default_language(MidgardConfig *self, const gchar *language)
-{
-	g_assert(self != NULL || language != NULL);
-
-	if(self->default_lang)
-		g_free(self->default_lang);
-
-	self->default_lang = g_strdup(language);
-}
-
 void midgard_config_set_loglevel(MidgardConfig *self, const gchar *loglevel)
 {
-	g_assert(self != NULL || loglevel != NULL);
+	g_assert(self != NULL && loglevel != NULL);
 
 	if(self->loglevel)
 		g_free(self->loglevel);
@@ -1428,7 +1381,7 @@ void midgard_config_set_testunit(MidgardConfig *self, gboolean toggle)
 
 void midgard_config_set_midgard_username(MidgardConfig *self, const gchar *username)
 {
-	g_assert(self != NULL || username != NULL);
+	g_assert(self != NULL && username != NULL);
 
 	if(self->mgdusername)
 		g_free(self->mgdusername);
@@ -1438,7 +1391,7 @@ void midgard_config_set_midgard_username(MidgardConfig *self, const gchar *usern
 
 void midgard_config_set_midgard_password(MidgardConfig *self, const gchar *password)
 {
-	g_assert(self != NULL || password != NULL);
+	g_assert(self != NULL && password != NULL);
 
 	if(self->mgdpassword)
 		g_free(self->mgdpassword);
@@ -1469,7 +1422,7 @@ void midgard_config_set_authtype(MidgardConfig *self, const gchar *authtype)
 
 void midgard_config_set_pamfile(MidgardConfig *self, const gchar *pamfile)
 {
-	g_assert(self != NULL || pamfile != NULL);
+	g_assert(self != NULL && pamfile != NULL);
 
 	if(self->pamfile)
 		g_free(self->pamfile);
@@ -1516,11 +1469,6 @@ _midgard_config_set_property (GObject *object, guint property_id,
 			self->logfilename = g_value_dup_string(value);
 			break;
 
-		case MIDGARD_CONFIG_DEFAULT_LANG:
-			g_free(self->default_lang);
-			self->default_lang = g_value_dup_string(value);
-			break;
-
 		case MIDGARD_CONFIG_LOGLEVEL:
 			g_free(self->loglevel);
 			self->loglevel = g_value_dup_string(value);
@@ -1616,10 +1564,6 @@ _midgard_config_get_property (GObject *object, guint property_id,
 			g_value_set_string (value, self->logfilename);
 			break;
 
-		case MIDGARD_CONFIG_DEFAULT_LANG:
-			g_value_set_string (value, self->default_lang);
-			break;
-
 		case MIDGARD_CONFIG_LOGLEVEL:
 			g_value_set_string(value, self->loglevel);
 			break;
@@ -1682,7 +1626,7 @@ _midgard_config_finalize(GObject *object)
 	
 	MidgardConfig *self = (MidgardConfig *) object;
 
-	__midgard_config_struct_free(self, TRUE);
+	__midgard_config_struct_free(self);
 }
 
 static void _midgard_config_class_init(
@@ -1754,15 +1698,6 @@ static void _midgard_config_class_init(
 			MIDGARD_CONFIG_LOGFILENAME,
 			pspec);
 
-	pspec = g_param_spec_string ("defaultlang",
-			"DefaultLanguage",  
-			"Default language",
-			"",
-			G_PARAM_READWRITE);
-	g_object_class_install_property (gobject_class,
-			MIDGARD_CONFIG_DEFAULT_LANG,
-			pspec);
-	
 	pspec = g_param_spec_string ("loglevel",
 			"Loglevel",
 			"Log level",
@@ -1798,7 +1733,7 @@ static void _midgard_config_class_init(
 	
 	pspec = g_param_spec_string ("midgardusername",
 			"MidgardUsername",
-			"Midgard sitegroup person's username",
+			"Midgard user's login",
 			"",
 			G_PARAM_READWRITE);
 	g_object_class_install_property (gobject_class,
@@ -1807,7 +1742,7 @@ static void _midgard_config_class_init(
 	
 	pspec = g_param_spec_string ("midgardpassword",
 			"MidgardPassword",
-			"Midgard sitegroup person's password",
+			"Midgard user's password",
 			"",
 			G_PARAM_READWRITE);   
 	g_object_class_install_property (gobject_class,
@@ -1876,14 +1811,10 @@ static void _midgard_config_instance_init(
 	MidgardConfig *self = (MidgardConfig *) instance;
 
 	__config_struct_new(self);
-
 	self->priv = midgard_config_private_new();
+	__set_dbtype (self, "MySQL");
 }
 
-/**
- * Returns the MidgardConfig value type. 
- * Registers the type as a fundamental GType unless already registered.
- */
 GType 
 midgard_config_get_type (void)
 {
