@@ -1,13 +1,13 @@
 
 #define MIDGARD_LIB_PREFIX "/usr/local"
-#define MIDGARD_GLOBAL_SCHEMA "/usr/local/share/midgard-2.0/MgdObjects.xml"
+#define MIDGARD_GLOBAL_SCHEMA "/usr/local/share/midgard2/MgdObjects.xml"
 
 /* Define directory path where all 'local' schemas are stored.
  * Global schema is basic midgard schema , while local ones are defined 
  * per virtual host or per application.
 */ 
-#define MIDGARD_LSCHEMA_DIR "/usr/local/share/midgard-2.0/schema"
-#define MIDGARD_SHARE_DIR "/usr/local/share/midgard-2.0" 
-#define MIDGARD_LIB_VERSION "9.03.0" 
+#define MIDGARD_LSCHEMA_DIR "/usr/local/share/midgard2/schema"
+#define MIDGARD_SHARE_DIR "/usr/local/share/midgard2" 
+#define MIDGARD_LIB_VERSION "9.09" 
 #define MIDGARD_LIB_VERSION_STRING ""
 #define MIDGARD_DBUS_SUPPORT 1
