@@ -70,17 +70,17 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 	MidgardCollector *mc;
 	guint _uint;
 	gint _int;
+	GObject *_object;
 
 	if(MIDGARD_IS_OBJECT(mgdobject)) {
 
 		gint object_action = -1;	
 
 		if(mgdobject->dbpriv->guid) {
+
 			GString *_sql = g_string_new(" ");
-			g_string_append_printf(_sql, "guid = '%s' "
-					"AND sitegroup = %d ",
-					mgdobject->dbpriv->guid,
-					mgdobject->dbpriv->sg);
+			g_string_append_printf(_sql, "guid = '%s' ",	
+					mgdobject->dbpriv->guid);
 			
 			gchar *tmpstr = g_string_free(_sql, FALSE);
 						
@@ -145,14 +145,7 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 			g_value_unset(&pval);
 			continue;
 		}
-		
-		if(klass && midgard_object_class_is_multilang(klass)) {
-			if(g_str_equal("lang", pspec[i]->name)) {
-				g_value_unset(&pval);
-				continue;	
-			}
-		}
-		
+				
 		/* Object is not fetched from database. Skip references */
 		if(mgdobject->dbpriv->guid == NULL)
 			goto export_unchecked_property;
@@ -260,36 +253,30 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 				gstring = g_string_new("");
 				g_string_append_printf(gstring,
 						"%d", g_value_get_int(&pval));
-				strprop = g_string_free(gstring, FALSE);
 				xmlNewChild(node, NULL, 
 						BAD_CAST pspec[i]->name,
-						BAD_CAST (xmlChar *)strprop);
-				g_free(strprop);
+						BAD_CAST (xmlChar *)gstring->str);
+				g_string_free (gstring, TRUE);
 				break;
 
 			case G_TYPE_UINT:
-				if(!g_str_equal(pspec[i]->name, 
-							"sitegroup")) {
-					gstring = g_string_new("");
-					g_string_append_printf(gstring, 
-							"%i", g_value_get_uint(&pval));
-					strprop = g_string_free(gstring, FALSE);
-					xmlNewChild(node, NULL,
-							BAD_CAST pspec[i]->name,
-							BAD_CAST (xmlChar *)strprop);
-					g_free(strprop);
-				}
+				gstring = g_string_new("");
+				g_string_append_printf(gstring,
+						"%d", g_value_get_uint(&pval));
+				xmlNewChild(node, NULL, 
+						BAD_CAST pspec[i]->name,
+						BAD_CAST (xmlChar *)gstring->str);
+				g_string_free (gstring, TRUE);
 				break;
 
 			case G_TYPE_FLOAT:
 				gstring = g_string_new("");
 				g_string_append_printf(gstring,
 						"%g", g_value_get_float(&pval));
-				strprop = g_string_free(gstring, FALSE);
 				xmlNewChild(node, NULL,
 						BAD_CAST pspec[i]->name,
-						BAD_CAST (xmlChar *)strprop);
-				g_free(strprop);
+						BAD_CAST (xmlChar *)gstring->str);
+				g_string_free (gstring, TRUE);
 				break;
 
 			case G_TYPE_BOOLEAN:
@@ -304,16 +291,41 @@ static void _write_nodes(GObject *object, xmlNodePtr node)
 				break;
 
 			case G_TYPE_OBJECT:
-				op_node = xmlNewNode(NULL, 
-						BAD_CAST pspec[i]->name);
-				_write_nodes(G_OBJECT(g_value_get_object(&pval)), 
-						op_node);
-				xmlAddChild(node, op_node);
+	
+				_object = g_value_get_object (&pval);
+				
+				if (_object) {
+
+					op_node = xmlNewNode(NULL, BAD_CAST pspec[i]->name);
+					_write_nodes(_object, op_node);
+					xmlAddChild(node, op_node);
+				}
 				break;
+
+			default:
+
+				if (pspec[i]->value_type == MIDGARD_TYPE_TIMESTAMP) {
+		
+					GValue strval = {0, };
+					g_value_init (&strval, G_TYPE_STRING);
+					g_value_transform (&pval, &strval);
+					xmlNewChild(node, NULL, 
+						BAD_CAST pspec[i]->name,
+						BAD_CAST (xmlChar *)g_value_get_string (&strval));
+					g_value_unset (&strval);
+
+
+				} else {
+					g_warning ("midgard_replicator_serialize: unhandled %s property type (%s)", 
+							pspec[i]->name, g_type_name (pspec[i]->value_type)); 
+				}
 		}
+
 		g_value_unset(&pval);
 	}
+
 	g_free(pspec);
+
 	if(mrp)
 		g_object_unref(mrp);
 }		
@@ -324,8 +336,6 @@ gchar *midgard_core_object_to_xml(GObject *object)
 	
 	xmlDocPtr doc = NULL; 
 	xmlNodePtr root_node = NULL;
-	gboolean multilang = FALSE;
-	gint lang;	
 		
 	LIBXML_TEST_VERSION;
 		
@@ -346,93 +356,6 @@ gchar *midgard_core_object_to_xml(GObject *object)
 	xmlDocSetRootElement(doc, root_node);
 	_write_nodes(G_OBJECT(object), object_node);
 
-	/* Add lang attribute if object's class is multilang */
-	if(MIDGARD_IS_OBJECT(object)) {
-		MidgardObjectClass *klass =
-			MIDGARD_OBJECT_GET_CLASS(object);
-		if(klass) {
-			if(midgard_object_class_is_multilang(klass)) {
-				multilang = TRUE;
-			}
-		}
-	}
-
-	guint current_object_lang = 0;
-
-	if(multilang) {
-	
-		MidgardConnection *_mgd = MGD_OBJECT_CNC(object);
-		
-		if(!_mgd) {
-
-			g_warning("MidgardConnection pointer not associated with '%s' instance",
-					G_OBJECT_TYPE_NAME(object));
-			return NULL;
-		}
-	
-		g_object_get(G_OBJECT(object), "lang", &current_object_lang, NULL);
-
-		guint i;
-		const gchar *lang_code;
-		/* Get current language. We will use it when all 
-		 * language contents are exported */
-		const gchar *init_lang = 
-			midgard_connection_get_lang(_mgd);
-		GObject **lang_objects = NULL;
-		guint n_objects;
-		if(MGD_OBJECT_GUID(object) != NULL)
-			lang_objects = midgard_object_get_languages(MIDGARD_OBJECT(object), &n_objects);
-
-		if(lang_objects) {
-			
-			for(i = 0; i < n_objects ; i++){
-
-				/* Get midgard_language object's id */
-				g_object_get(lang_objects[i], "id", &lang, NULL);
-
-				/* Do not create duplicates in xml */
-				if(lang == current_object_lang)
-					continue;
-
-				g_object_get(lang_objects[i], "code", &lang_code, NULL);
-
-				midgard_connection_set_lang(_mgd, lang_code);
-				
-				GValue gval = {0, };
-				g_value_init(&gval , G_TYPE_STRING);
-				g_value_set_string(&gval, MGD_OBJECT_GUID(object));
-				MgdObject *lobject = 
-					midgard_object_new(_mgd, 
-							G_OBJECT_TYPE_NAME(G_OBJECT(object)),
-							&gval);
-				g_value_unset(&gval);
-
-				object_node =
-					xmlNewNode(NULL, 
-							BAD_CAST G_OBJECT_TYPE_NAME(
-								G_OBJECT(object)));
-			
-				gchar *code;
-				g_object_get(lang_objects[i], "code", &code, NULL);
-				xmlNewProp(object_node,
-						BAD_CAST "lang", BAD_CAST code);
-				g_free(code);
-				/* Add purged info */
-				xmlNewProp(object_node, BAD_CAST "purge", BAD_CAST "no");
-				xmlAddChild(root_node, object_node);
-				_write_nodes(G_OBJECT(lobject), object_node);
-
-				g_object_unref(lobject);
-				g_object_unref(lang_objects[i]);
-			}
-		}
-
-		g_free(lang_objects);
-
-		/* Set language to its initial state */
-		midgard_connection_set_lang(_mgd, init_lang);
-	}
-
 	xmlChar *buf;
 	gint size;
 	xmlDocDumpFormatMemoryEnc(doc, &buf, &size, "UTF-8", 1);
@@ -820,20 +743,6 @@ GObject **midgard_core_object_from_xml(MidgardConnection *mgd,
 				if(MIDGARD_IS_BLOB(object)) 
 					MIDGARD_BLOB(object)->priv->parentguid = g_strdup((gchar *)attr);
 
-				xmlFree(attr);
-			}
-			/* Set lang */
-			guint lang = 0;
-			attr = xmlGetProp(child, BAD_CAST "lang");
-			if(attr) {
-				if(!g_str_equal(attr, "") && MIDGARD_IS_DBOBJECT(object)){
-
-					lang = midgard_connection_get_lang_id(
-							MIDGARD_DBOBJECT(object)->dbpriv->mgd,
-							(const gchar *)attr);					
-				}
-				g_object_set(object, "lang", lang, NULL);
-
 				xmlFree(attr);
 			}
 		}
@@ -858,22 +767,3 @@ GObject **midgard_core_object_from_xml(MidgardConnection *mgd,
 
 	return (GObject **) _objects;
 }
-
-gboolean _midgard_object_violates_sitegroup(MgdObject *object)
-{
-	g_assert(object != NULL);
-	g_assert(object->dbpriv->mgd != NULL);
-
-	MidgardConnection *mgd = object->dbpriv->mgd;
-	MidgardUser *user = MIDGARD_USER(mgd->priv->user);
-
-	if(object->dbpriv->sg == 0
-			&& !midgard_user_is_root(user)){
-
-		MIDGARD_ERRNO_SET(object->dbpriv->mgd,
-				MGD_ERR_SITEGROUP_VIOLATION);
-		return TRUE;
-	}
-
-	return FALSE;
-}
