@@ -25,6 +25,7 @@
 #include "midgard_core_object_class.h"
 #include "midgard_object_class.h"
 #include "midgard_error.h"
+#include "midgard_core_connection.h"
 #ifdef HAVE_LIBGDA_4
 #include <sql-parser/gda-sql-parser.h>
 #endif
@@ -34,7 +35,7 @@
  */ 
 #define _DB_DEFAULT_DATETIME "0001-01-01 00:00:00"
 
-static void __check_property_index(MidgardObjectClass *klass, MidgardReflectionProperty *mrp,
+static void __check_property_index(MidgardDBObjectClass *klass, MidgardReflectionProperty *mrp,
 		const gchar *property, MidgardDBColumn *mdc);
 
 MidgardDBColumn *midgard_core_dbcolumn_new(void) 
@@ -57,14 +58,12 @@ MidgardDBColumn *midgard_core_dbcolumn_new(void)
 }
 
 gchar *midgard_core_query_where_guid(
-		const gchar *table, const gchar *guid, guint sitegroup)
+		const gchar *table, const gchar *guid)
 {
 	g_assert(guid != NULL);
 
 	GString *where = g_string_new("");
-	g_string_append_printf(where, 
-			"%s.guid = '%s' AND %s.sitegroup = %d", 
-			table, guid, table, sitegroup);
+	g_string_append_printf(where, "%s.guid = '%s' ", table, guid);
 
 	return g_string_free(where, FALSE);
 }
@@ -212,11 +211,32 @@ GValue *midgard_core_query_get_field_value(
 	g_free(query);
 
 	if(!model)
+	       return NULL;
+	
+	if (!GDA_IS_DATA_MODEL (model)) {
+	
+		g_warning ("Returned model is not GDA_DATA_MODEL");
+		MIDGARD_ERRNO_SET (mgd, MGD_ERR_INTERNAL);
 		return NULL;
 
+	}
+
 	const GValue *value =
 		midgard_data_model_get_value_at(model, 0, 0);
 
+	if (!value) {
+
+		g_object_unref (model);
+		return NULL;
+	}
+
+	if (G_VALUE_TYPE (value) == GDA_TYPE_NULL) {
+
+		g_object_unref (model);
+		g_warning ("Unexpected NULL type for returned value");
+		return NULL;
+	}
+
 	GValue *new_value = g_new0(GValue, 1);
 	g_value_init(new_value, G_VALUE_TYPE(value));
 	g_value_copy(value, new_value);
@@ -226,15 +246,15 @@ GValue *midgard_core_query_get_field_value(
 	return new_value;
 }
 
-guint midgard_core_query_get_id(MidgardConnection *mgd,
-		const gchar *table, const gchar *guid, guint sitegroup)
+guint 
+midgard_core_query_get_id (MidgardConnection *mgd, const gchar *table, const gchar *guid)
 {
 	g_assert(mgd != NULL);
 	g_assert(table != NULL);
 	g_assert(guid != NULL);
 
 	gchar *where = 
-		midgard_core_query_where_guid(table, guid, sitegroup);
+		midgard_core_query_where_guid(table, guid);
 
 	GValue *value = 
 		midgard_core_query_get_field_value(
@@ -272,6 +292,410 @@ guint midgard_core_query_get_id(MidgardConnection *mgd,
 	return id;
 }
 
+static void 
+_add_value_type (GString *str, GParameter parameter, gboolean add_comma)
+{
+	if (add_comma) 
+		g_string_append (str, ", ");
+
+	const gchar *type = "invalid";
+
+	switch (G_TYPE_FUNDAMENTAL (G_VALUE_TYPE(&parameter.value))) {
+		
+		case G_TYPE_STRING:
+			type = "gchararray";
+		break;
+	
+		case G_TYPE_UINT:
+			type = "guint";
+		break;
+
+		case G_TYPE_INT:
+			type = "gint";
+		break;
+
+		case G_TYPE_BOOLEAN:
+			type = "gboolean";
+		break;	
+	}
+
+	g_string_append_printf (str, "##/*name:'%s' type:%s*/", parameter.name, type);
+}
+
+GdaDataModel *
+midgard_core_query_get_dbobject_model (MidgardConnection *mgd, MidgardDBObjectClass *klass, guint n_params, const GParameter *parameters)
+{
+	g_return_val_if_fail (mgd != NULL, NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (mgd->priv->connection), NULL);
+	g_return_val_if_fail (klass != NULL, NULL);
+
+	MgdSchemaTypeAttr *type_attr = midgard_core_class_get_type_attr (klass);
+	g_return_val_if_fail (type_attr->sql_select_full != NULL, NULL);
+
+	const gchar *table = midgard_core_class_get_table (klass);
+	g_return_val_if_fail (table != NULL, NULL);
+
+	GString *select = g_string_new ("SELECT ");
+	g_string_append_printf (select, "%s FROM %s WHERE ", type_attr->sql_select_full, table);
+
+	guint i;
+	GString *where = g_string_new("");
+	for (i = 0; i < n_params; i++) {
+	
+		g_string_append_printf (where, "%s %s = ", i > 0 ? " AND " : "", parameters[i].name);
+		_add_value_type (where, parameters[i], FALSE);
+	}
+
+	g_string_append_printf (select, " %s ", where->str);
+
+	GdaDict *dict = gda_dict_new ();
+	gda_dict_set_connection (dict, mgd->priv->connection);
+
+	GdaQuery *query = gda_query_new_from_sql (dict, select->str, NULL);
+
+	g_string_free (select, TRUE);
+	g_string_free (where, TRUE);
+
+        if (!query) {
+		g_warning ("Failed to create GdaQuery from sql");
+		return NULL;
+	}
+
+	GdaParameter *param = NULL;
+        GdaParameterList *plist = gda_query_get_parameter_list (query);
+
+	for (i = 0; i < n_params; i++) {
+
+        	param = gda_parameter_list_find_param (plist, parameters[i].name);
+		gda_parameter_set_value (param, (const GValue *) &parameters[i].value);
+	}
+
+	gda_connection_clear_events_list (gda_dict_get_connection (dict));
+
+	GError *error = NULL;
+	GdaObject *exec_res = gda_query_execute (query, plist, FALSE, &error);
+
+	g_debug("%s", gda_query_get_sql_text(query));
+
+	if (!exec_res) {
+		
+		g_warning ("%s. Query %s failed", error && error->message ? error->message : "Unknown reason", gda_query_get_sql_text (query));
+		g_object_unref (dict);
+		g_object_unref (query);
+		return NULL;
+	}
+
+	g_object_unref (query);
+	g_object_unref (dict);	
+
+	if (!GDA_IS_DATA_MODEL (exec_res)) {
+
+		g_warning ("Failed to retrieve data model from query");
+		return NULL;
+	}
+	
+	return GDA_DATA_MODEL (exec_res);	
+}
+
+gboolean 
+midgard_core_query_create_dbobject_record (MidgardDBObject *object)
+{
+	g_return_val_if_fail (object != NULL, FALSE);
+	
+	GdaConnection *cnc = (MGD_OBJECT_CNC (object))->priv->connection;
+	gboolean bv;
+
+	MidgardDBObjectClass *klass = g_type_class_peek (G_OBJECT_TYPE (object));
+	if (!klass) {
+
+		g_warning ("Can not find class pointer for %s instance", G_OBJECT_TYPE_NAME (object));
+		return FALSE;
+	}
+
+	const gchar *table = midgard_core_class_get_table (klass);
+	if (!table) {
+		
+		g_warning ("Empty table for %s class", G_OBJECT_TYPE_NAME (object));
+		return FALSE;
+	}
+
+	guint n_prop;
+	guint i;
+	GParamSpec **pspecs = g_object_class_list_properties (G_OBJECT_CLASS (klass), &n_prop);
+
+	if (!pspecs) {
+		
+		g_warning ("Properties not registered for %s class", G_OBJECT_TYPE_NAME (object));
+		return FALSE;
+	}
+
+	GdaDict *dict = gda_dict_new ();
+        gda_dict_set_connection (dict , (MGD_OBJECT_CNC (object))->priv->connection);
+
+	GString *sqlquery = g_string_new ("INSERT INTO ");
+	g_string_append_printf (sqlquery, "%s ( ", table);
+
+	GString *values = g_string_new (" VALUES( ");
+	MgdSchemaPropertyAttr *prop_attr = NULL;
+
+	/* Create parameters from properties */
+	GParameter *parameters = g_new0 (GParameter, n_prop);
+
+	for (i = 0; i < n_prop; i++) {
+		
+		GValue value = {0, };
+		g_value_init (&value, pspecs[i]->value_type);
+		g_object_get_property (G_OBJECT (object), pspecs[i]->name, &value);
+
+		/* Convert boolean to integer, it's safe for SQLite at least */
+		if (pspecs[i]->value_type == G_TYPE_BOOLEAN) {
+
+			bv = g_value_get_boolean (&value);
+			g_value_unset (&value);
+
+			g_value_init (&value, G_TYPE_UINT);
+			g_value_set_uint (&value, bv ? 1 : 0);
+		}
+
+		/* Map property to storage field */
+		prop_attr = midgard_core_class_get_property_attr (klass, pspecs[i]->name);
+		parameters[i].name = prop_attr->field;
+		parameters[i].value = (const GValue)value;
+		g_string_append_printf (sqlquery, "%s %s", i > 0 ? "," : "", prop_attr->field);
+		_add_value_type (values, parameters[i], i > 0 ? TRUE : FALSE);
+		g_value_unset (&value);
+	}
+
+	g_string_append_printf (sqlquery, " ) %s )", values->str);
+
+	GError *error = NULL;
+        GdaQuery *query = gda_query_new_from_sql (dict, sqlquery->str, &error);
+
+	g_string_free (sqlquery, TRUE);
+	g_string_free (values, TRUE);
+
+	if (query) 
+		g_debug("%s", gda_query_get_sql_text(query));
+
+	if (!query || error) {
+		g_warning ("Failed to create new query from string. %s", error && error->message ? error->message : "Unknown reason");
+
+		if (query)
+			g_object_unref (query);
+
+		g_object_unref (dict);
+		g_free (pspecs);
+		
+		return FALSE;
+	}
+
+	GdaParameterList *plist = gda_query_get_parameter_list (query);
+	GdaParameter *param;
+
+	for (i = 0; i < n_prop; i++) {
+
+		/* Map property to storage field */
+		prop_attr = midgard_core_class_get_property_attr (klass, pspecs[i]->name);
+		param = gda_parameter_list_find_param (plist, prop_attr->field);
+		GValue val = {0, };
+		g_value_init (&val, pspecs[i]->value_type);
+		g_object_get_property (G_OBJECT(object), pspecs[i]->name, &val);
+
+		/* Convert boolean to integer, it's safe for SQLite at least */
+		if (pspecs[i]->value_type == G_TYPE_BOOLEAN) {
+
+			bv = g_value_get_boolean (&val);
+			g_value_unset (&val);
+
+			g_value_init (&val, G_TYPE_UINT);
+			g_value_set_uint (&val, bv ? 1 : 0);
+		}
+
+		gda_parameter_set_value (param, (const GValue*) &val);
+		g_value_unset (&val);
+	}
+
+	gda_connection_clear_events_list (cnc);
+
+	g_free (pspecs);
+	g_object_unref (dict);
+
+	if (error)
+		g_clear_error (&error);
+	error = NULL;
+
+	/* TODO , debug query */
+	GdaObject *res_object = gda_query_execute (query, plist, FALSE, &error);
+
+	g_object_unref (query);
+	g_object_unref (plist);
+
+	if (!res_object)
+		return FALSE;
+
+	g_object_unref (res_object);
+
+	if (error) {
+
+		g_warning ("Failed to create object record. %s", error && error->message ? error->message : "Unknown reason");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+gboolean 
+midgard_core_query_update_dbobject_record (MidgardDBObject *object)
+{
+	g_return_val_if_fail (object != NULL, FALSE);
+	
+	GdaConnection *cnc = (MGD_OBJECT_CNC (object))->priv->connection;
+	g_return_val_if_fail (cnc != NULL, FALSE);
+
+	MidgardDBObjectClass *klass = g_type_class_peek (G_OBJECT_TYPE (object));
+	if (!klass) {
+
+		g_warning ("Can not find class pointer for %s instance", G_OBJECT_TYPE_NAME (object));
+		return FALSE;
+	}
+
+	const gchar *table = midgard_core_class_get_table (klass);
+	if (!table) {
+		
+		g_warning ("Empty table for %s class", G_OBJECT_TYPE_NAME (object));
+		return FALSE;
+	}
+
+	guint n_prop;
+	guint i;
+	GParamSpec **pspecs = g_object_class_list_properties (G_OBJECT_CLASS (klass), &n_prop);
+
+	if (!pspecs) {
+		
+		g_warning ("Properties not registered for %s class", G_OBJECT_TYPE_NAME (object));
+		return FALSE;
+	}
+
+	const gchar *guid = MGD_OBJECT_GUID (object);
+	if (!guid || (guid && *guid == '\0')) {
+
+		g_warning ("an not update object with empty guid");
+		return FALSE;
+	}
+
+	GdaDict *dict = gda_dict_new ();
+        gda_dict_set_connection (dict , (MGD_OBJECT_CNC (object))->priv->connection);
+
+	GString *sqlquery = g_string_new ("UPDATE ");
+	g_string_append_printf (sqlquery, "%s SET ", table);
+
+	MgdSchemaPropertyAttr *prop_attr = NULL;
+	gboolean bv;
+
+	/* Create parameters from properties */
+	GParameter *parameters = g_new0 (GParameter, n_prop);
+
+	for (i = 0; i < n_prop; i++) {
+		
+		GValue value = {0, };
+		g_value_init (&value, pspecs[i]->value_type);
+		g_object_get_property (G_OBJECT (object), pspecs[i]->name, &value);
+		/* Map property to storage field */
+		prop_attr = midgard_core_class_get_property_attr (klass, pspecs[i]->name);
+
+		/* Convert boolean to integer, it's safe for SQLite at least */
+		if (pspecs[i]->value_type == G_TYPE_BOOLEAN) {
+
+			bv = g_value_get_boolean (&value);
+			g_value_unset (&value);
+
+			g_value_init (&value, G_TYPE_UINT);
+			g_value_set_uint (&value, bv ? 1 : 0);
+		}
+
+		parameters[i].name = prop_attr->field;
+		parameters[i].value = (const GValue)value;
+		g_string_append_printf (sqlquery, "%s %s=", i > 0 ? "," : "", prop_attr->field);
+		_add_value_type (sqlquery, parameters[i], FALSE);
+		g_value_unset (&value);
+	}
+
+	g_string_append_printf (sqlquery, " WHERE %s.guid = '%s'", table, guid);  	
+
+	GError *error = NULL;
+        GdaQuery *query = gda_query_new_from_sql (dict, sqlquery->str, &error);
+
+	g_string_free (sqlquery, TRUE);
+
+	if (query) 
+		g_debug ("%s", gda_query_get_sql_text (query));
+
+        if (!query || error) {
+		
+		g_warning ("Failed to create new query from string. %s", error && error->message ? error->message : "Unknown reason");
+
+		if (query)
+			g_object_unref (query);
+
+		g_object_unref (dict);
+		g_free (pspecs);
+		
+		return FALSE;
+	}
+
+	GdaParameterList *plist = gda_query_get_parameter_list (query);
+	GdaParameter *param;
+
+	for (i = 0; i < n_prop; i++) {
+
+		/* Map property to storage field */
+		prop_attr = midgard_core_class_get_property_attr (klass, pspecs[i]->name);
+		param = gda_parameter_list_find_param (plist, prop_attr->field);
+		GValue val = {0, };
+		g_value_init (&val, pspecs[i]->value_type);
+		g_object_get_property (G_OBJECT(object), pspecs[i]->name, &val);
+		
+		/* Convert boolean to integer, it's safe for SQLite at least */
+		if (pspecs[i]->value_type == G_TYPE_BOOLEAN) {
+
+			bv = g_value_get_boolean (&val);
+			g_value_unset (&val);
+
+			g_value_init (&val, G_TYPE_UINT);
+			g_value_set_uint (&val, bv ? 1 : 0);
+		}
+
+		gda_parameter_set_value (param, (const GValue*) &val);
+	}
+
+	gda_connection_clear_events_list (cnc);
+
+	g_free (pspecs);
+	g_object_unref (dict);
+
+	if (error)
+		g_clear_error (&error);
+	error = NULL;
+
+	/* TODO , debug query */
+	GdaObject *res_object = gda_query_execute (query, plist, FALSE, &error);
+
+	g_object_unref (query);
+	g_object_unref (plist);
+
+	if (!res_object)
+		return FALSE;
+
+	g_object_unref (res_object);
+
+	if (error) {
+
+		g_warning ("Failed to update object record. %s", error && error->message ? error->message : "Unknown reason");
+		return FALSE;
+	}
+
+	return TRUE;
+}
 gboolean midgard_core_query_update_object_fields(MidgardDBObject *object, const gchar *field, ...)
 {
 	g_assert(object != NULL);
@@ -292,13 +716,9 @@ gboolean midgard_core_query_update_object_fields(MidgardDBObject *object, const
 	va_end(var_args);	
 
 	MidgardConnection *mgd = MGD_OBJECT_CNC(object);
-	const gchar *table = 
-		midgard_core_class_get_table(MIDGARD_DBOBJECT_GET_CLASS(object));
+	const gchar *table = midgard_core_class_get_table(MIDGARD_DBOBJECT_GET_CLASS(object));
 
-	gchar *where =
-		midgard_core_query_where_guid(table, 
-				MGD_OBJECT_GUID(object), 
-				MGD_OBJECT_SG(object));
+	gchar *where = midgard_core_query_where_guid(table, MGD_OBJECT_GUID(object));
 
 #ifdef HAVE_LIBGDA_4
 	gint rv = midgard_core_query_insert_records(mgd, table, 
@@ -311,6 +731,8 @@ gboolean midgard_core_query_update_object_fields(MidgardDBObject *object, const
 	g_slist_free(cols);
 	g_slist_free(values);
 
+	g_free (where);
+
 	if(rv == 0)
 		return TRUE;
 
@@ -505,12 +927,12 @@ gint midgard_core_query_insert_records(MidgardConnection *mgd,
 		
 			/* GDA3 & SQLite workaround */
 			if (mgd->priv->config->priv->dbtype == MIDGARD_DB_TYPE_SQLITE)
-				value = gda_query_field_value_new(query, G_TYPE_STRING);
+				value = gda_query_field_value_new (query, G_TYPE_STRING);
 			else 
-				value = gda_query_field_value_new(query, GDA_TYPE_TIMESTAMP);
+				value = gda_query_field_value_new (query, GDA_TYPE_TIMESTAMP);
+
+			gda_query_field_value_set_value (GDA_QUERY_FIELD_VALUE (value), &tval);
 
-			gda_query_field_value_set_value(
-					GDA_QUERY_FIELD_VALUE(value), &tval);
 			g_value_unset(&tval);
 
 		} else if (G_VALUE_TYPE(val) == G_TYPE_STRING
@@ -551,24 +973,27 @@ gint midgard_core_query_insert_records(MidgardConnection *mgd,
 		values = values->next;
 	}
 
+	gchar *full_query = NULL;
+
 	if(where != NULL) {		
 		
-		GdaQueryCondition *cond = 
-			gda_query_condition_new_from_sql(query, where, NULL, NULL);
-		gda_query_set_condition (query, cond);
+		const gchar *sql = gda_query_get_sql_text (query);
+		full_query = g_strconcat (sql, " WHERE ", where, NULL);
+		gda_query_set_sql_text (query, full_query, NULL);
 	}
 
-	/* TODO, refactor this debug log to avoid 
-	 * another one SQL rendering ( done in libgda ) */
-	const gchar *debug_query = gda_query_get_sql_text(query);
-	g_debug("%s", debug_query);
+	if (full_query) {
+
+		g_debug ("%s", full_query);
+		g_free (full_query);
+	}
 
 	GError *error = NULL;
 	gda_query_execute(query, NULL, FALSE, &error);
 
 	g_object_unref(target);
 	g_object_unref(dict);
-	g_object_unref(query);
+	g_object_unref(query); 
 	
 	if(error){
 		g_clear_error(&error);
@@ -613,7 +1038,7 @@ gboolean __table_exists(MidgardConnection *mgd, const gchar *tablename)
 		retval = FALSE;
 	g_object_unref (dm_schema);
 	return retval;
-#else
+#else	
 	GdaDataModel *dm_schema =
 		gda_connection_get_schema(mgd->priv->connection,
 				GDA_CONNECTION_SCHEMA_TABLES, NULL ,NULL);
@@ -648,6 +1073,9 @@ gboolean midgard_core_table_exists(MidgardConnection *mgd, const gchar *tablenam
 gboolean midgard_core_query_create_table(MidgardConnection *mgd, 
 		const gchar *descr, const gchar *tablename, const gchar *primary)
 {
+	if (primary == NULL)
+		primary = "id";
+
 	g_assert(mgd != NULL);
 	g_assert(tablename != NULL);
 
@@ -689,11 +1117,7 @@ gboolean midgard_core_query_create_table(MidgardConnection *mgd,
 	xmlDocSetRootElement (parameters, root);
 
 	table = xmlNewChild (root, NULL, (xmlChar*)"op_data", (xmlChar*)tablename);
-	xmlSetProp(table, (xmlChar*)"path", (xmlChar*) "/TABLE_DEF_P/TABLE_NAME");
-	
-	op_data = xmlNewChild (root, NULL, (xmlChar*)"op_data", NULL);
-	xmlSetProp(op_data, (xmlChar*)"path", (xmlChar*)"/FIELDS_A");
-	array_data = xmlNewChild (op_data, NULL, (xmlChar*)"gda_array_data", NULL);
+	xmlSetProp(table, (xmlChar*)"path", (xmlChar*) "/TABLE_DEF_P/TABLE_NAME");	
 
 	/* Default columns */	
 	/* guid column */
@@ -716,22 +1140,14 @@ gboolean midgard_core_query_create_table(MidgardConnection *mgd,
 				(xmlChar*)"gda_array_value", (xmlChar*)"true");
 		xmlSetProp(array_value, (xmlChar*)"colid", (xmlChar*)"COLUMN_NNUL");
 	}*/
-
-	/* sitegroup column */
-	/*array_row = xmlNewChild (array_data, NULL, (xmlChar*)"gda_array_row", NULL);
-	array_value = xmlNewChild (array_row, NULL, 
-			(xmlChar*)"gda_array_value", (xmlChar*)"sitegroup");
-	xmlSetProp(array_value, (xmlChar*)"colid", (xmlChar*)"COLUMN_NAME");
-	array_value = xmlNewChild(array_row, NULL, 
-			(xmlChar*)"gda_array_value", (xmlChar*)"int");
-	xmlSetProp(array_value, (xmlChar*)"colid", (xmlChar*)"COLUMN_TYPE");
-	array_value = xmlNewChild(array_row, NULL,
-			(xmlChar*)"gda_array_value", (xmlChar*)"0");
-	xmlSetProp(array_value, (xmlChar*)"colid", (xmlChar*)"COLUMN_DEFAULT");
-	*/
 	
 	/* Add primary field if defined */
 	if(primary) {
+
+		op_data = xmlNewChild (root, NULL, (xmlChar*)"op_data", NULL);
+		xmlSetProp(op_data, (xmlChar*)"path", (xmlChar*)"/FIELDS_A");
+		array_data = xmlNewChild (op_data, NULL, (xmlChar*)"gda_array_data", NULL);
+
 		array_row = 
 			xmlNewChild (array_data, NULL, (xmlChar*)"gda_array_row", NULL);
 		array_value = xmlNewChild (array_row, NULL, 
@@ -759,7 +1175,7 @@ gboolean midgard_core_query_create_table(MidgardConnection *mgd,
 		xmlSetProp(array_value, (xmlChar*)"colid", (xmlChar*)"COLUMN_AUTOINC");
 	}
 
-	if(!gda_server_operation_load_data_from_xml (op, root, &error)){
+	if (!gda_server_operation_load_data_from_xml (op, root, &error)){
 		
 		g_warning("Can not load table data from xml. %s", error->message);
 		xmlFreeDoc(parameters);
@@ -884,7 +1300,6 @@ gboolean midgard_core_query_add_column(MidgardConnection *mgd,
 
 	const gchar *column_name = mdc->column_name;
 	const gchar *table_name = mdc->table_name;
-	const gchar *column_type_str = mdc->dbtype;
 
 	if (mdc->dbtype == NULL) {
 		
@@ -894,8 +1309,10 @@ gboolean midgard_core_query_add_column(MidgardConnection *mgd,
 	if(g_str_equal(mdc->dbtype, "longtext"))
 		mdc->dbtype = "text";
 
-	if((mgd->priv->config->priv->dbtype == MIDGARD_DB_TYPE_POSTGRES)
-			&& (g_str_equal(mdc->dbtype,"datetime"))) {
+	const gchar *column_type_str;
+	if ((mgd->priv->config->priv->dbtype == MIDGARD_DB_TYPE_POSTGRES)
+			&& (g_str_equal(mdc->dbtype,"datetime"))
+	) {
 
 		column_type_str = "timestamp";
 
@@ -1061,11 +1478,15 @@ gboolean __index_exists(MidgardConnection *mgd, MidgardDBColumn *mdc, const gcha
 	guint j;
 	const GValue *value;
 	gboolean index_exists = FALSE;
-	
+
 	for(j = 0; j < rows; j++) {
-		
+	
+		value = gda_data_model_get_value_at (dm_schema, 0, j);
+		if (!g_str_equal(mdc->column_name, g_value_get_string (value)))
+			continue;
+
 		value = gda_data_model_get_value_at(dm_schema, 4, j);
-		index_exists = g_value_get_boolean(value);
+		index_exists = g_value_get_boolean (value);
 		
 		if (index_exists) {
 			
@@ -1086,11 +1507,12 @@ gboolean midgard_core_query_add_index(MidgardConnection *mgd,
 	g_return_val_if_fail(mdc->table_name != NULL, FALSE);
 	g_return_val_if_fail(mdc->column_name != NULL, FALSE);
 
-	if(!mdc->index)
-		g_debug("Index not defined for %s.%s. Not created",
-				mdc->table_name, mdc->column_name);
+	if(!mdc->index) {
+		g_debug("Index not defined for %s.%s. Not created", mdc->table_name, mdc->column_name);
+		return TRUE;
+	}
 
-	gchar *index_name = g_strconcat(mdc->column_name, "_idx", NULL);
+	gchar *index_name = g_strconcat(mdc->table_name, "_", mdc->column_name, "_idx", NULL);
 	if (__index_exists(mgd, mdc, (const gchar *)index_name)) {
 
 		g_free(index_name);
@@ -1129,7 +1551,7 @@ gboolean midgard_core_query_add_index(MidgardConnection *mgd,
 
 	if(mdc->index) {
 		
-		index_name = g_strconcat(mdc->column_name, "_idx", NULL);
+		index_name = g_strconcat(mdc->table_name, "_", mdc->column_name, "_idx", NULL);
 
 		gda_server_operation_set_value_at(op, index_name,
 				NULL, "/INDEX_DEF_P/INDEX_NAME");
@@ -1147,14 +1569,21 @@ gboolean midgard_core_query_add_index(MidgardConnection *mgd,
 		gchar *_sql = 
 			gda_server_provider_render_operation(server, cnc, op, NULL);
 		g_debug("Rendering create index: %s", _sql);
-		
+	
+		/* Disconnect default error callback.
+		   This is a hack, there's no way to check if index exists, so we make error be silent */
+		midgard_core_connection_disconnect_error_callback (mgd);
+	
 		gboolean created =
 			gda_server_provider_perform_operation (server, cnc, op, &error);
 		
+		/* Connect default error callback again */
+		midgard_core_connection_connect_error_callback (mgd);
+
 		if(!created) {
 			
-			g_warning("Can not create index on %s.%s ( %s )",
-					column_name, table_name, error->message);
+			g_debug("Can not create index on %s.%s ( %s )",
+					column_name, table_name, error->message); 
 			g_clear_error(&error);
 			g_object_unref(op);
 			return FALSE;
@@ -1288,7 +1717,7 @@ static gboolean __create_columns(MidgardConnection *mgd, xmlNode *tbln)
 gboolean midgard_core_query_create_basic_db(MidgardConnection *mgd)
 {
 	g_return_val_if_fail(mgd != NULL, FALSE);
-
+	
 	gchar *dbxml = g_build_path(G_DIR_SEPARATOR_S,
 			mgd->priv->config->sharedir, "midgard_initial_database.xml", NULL);
 
@@ -1480,7 +1909,7 @@ gboolean midgard_core_query_create_metadata_columns(
 	return TRUE;
 }
 
-void __check_property_index(MidgardObjectClass *klass, MidgardReflectionProperty *mrp,
+void __check_property_index(MidgardDBObjectClass *klass, MidgardReflectionProperty *mrp,
 		const gchar *property, MidgardDBColumn *mdc) 
 {
 	g_assert(klass != NULL);
@@ -1490,12 +1919,6 @@ void __check_property_index(MidgardObjectClass *klass, MidgardReflectionProperty
 	if(mdc->index == TRUE)
 		return;
 
-	if (g_str_equal(property, "sitegroup")) {
-	
-		mdc->index = TRUE;
-		return;
-	}
-
 	if (midgard_reflection_property_is_link(mrp, property)) {
 		
 		mdc->index = TRUE;
@@ -1508,17 +1931,18 @@ void __check_property_index(MidgardObjectClass *klass, MidgardReflectionProperty
 		return;
 	}
 
-	const gchar *parent =
-		midgard_object_class_get_property_parent(klass);
+	const gchar *parent = NULL;
+	const gchar *up = NULL;
+	if (MIDGARD_IS_OBJECT_CLASS (klass)) {
+		parent = midgard_object_class_get_property_parent(MIDGARD_OBJECT_CLASS(klass));
+		up = midgard_object_class_get_property_up(MIDGARD_OBJECT_CLASS(klass));	
+	}
 
 	if (parent && g_str_equal(parent, property)) {
 
 		mdc->index = TRUE;
 		return;
 	}
-
-	const gchar *up =
-		midgard_object_class_get_property_up(klass);
 	
 	if(up && g_str_equal(up, property)) {
 
@@ -1526,15 +1950,6 @@ void __check_property_index(MidgardObjectClass *klass, MidgardReflectionProperty
 		return;
 	}
 
-	if (midgard_object_class_is_multilang(klass)) {
-		
-		if(g_str_equal(property, "sid")) {
-
-			mdc->index = TRUE;
-			return;
-		}
-	}
-
 	MgdSchemaPropertyAttr *prop_attr;
 	prop_attr = g_hash_table_lookup(klass->dbpriv->storage_data->prophash, property);
 	
@@ -1545,15 +1960,20 @@ void __check_property_index(MidgardObjectClass *klass, MidgardReflectionProperty
 	}
 }
 
-gboolean midgard_core_query_update_class_storage(MidgardConnection *mgd, MidgardObjectClass *klass)
+gboolean midgard_core_query_update_class_storage(MidgardConnection *mgd, MidgardDBObjectClass *klass)
 {
 	g_return_val_if_fail(mgd != NULL, FALSE);
 	g_return_val_if_fail(klass != NULL, FALSE);
 
-	const gchar *tablename =
-		midgard_core_class_get_table(MIDGARD_DBOBJECT_CLASS(klass));
-	const gchar *classname =
-		G_OBJECT_CLASS_NAME(G_OBJECT_CLASS(klass));
+	const gchar *tablename = midgard_core_class_get_table(klass);
+	const gchar *classname = G_OBJECT_CLASS_NAME(G_OBJECT_CLASS(klass));
+
+	/* Class with empty storage, ignore as silently as possible */
+	if (!tablename) {
+		g_debug("Can not update %s class table. None defined.", classname);
+		return TRUE;
+	}
+
 	guint n_prop, i = 0;
 
 	if(!__table_exists(mgd, tablename)) {
@@ -1583,7 +2003,7 @@ gboolean midgard_core_query_update_class_storage(MidgardConnection *mgd, Midgard
 	
 		nick = midgard_core_class_get_property_tablefield
 			(MIDGARD_DBOBJECT_CLASS(klass), pspecs[i]->name);
-	
+
 		if (nick == NULL)
 			continue;
 		
@@ -1592,18 +2012,10 @@ gboolean midgard_core_query_update_class_storage(MidgardConnection *mgd, Midgard
 		
 		spltd = g_strsplit(nick, ".", 0);
 		
-		/* ML class */
-		if(midgard_object_class_is_multilang(klass)) {
-			mdc->column_name = spltd[1];
-			mdc->table_name = spltd[0];
-		} else {
-			mdc->column_name = spltd[1];
-			mdc->table_name = tablename;
-		}
-
-		mgd_type = midgard_reflection_property_get_midgard_type(
-				mrp, pspecs[i]->name);
+		mdc->column_name = spltd[1];
+		mdc->table_name = tablename;
 
+		mgd_type = midgard_reflection_property_get_midgard_type (mrp, pspecs[i]->name);
 		mdc->gtype = mgd_type;
 		
 		switch(pspecs[i]->value_type) {
@@ -1669,11 +2081,21 @@ gboolean midgard_core_query_update_class_storage(MidgardConnection *mgd, Midgard
 	g_object_unref(mrp);
 	g_free(pspecs);
 
+	if (klass->dbpriv->has_metadata) {
+	
+		if(!midgard_core_query_create_metadata_columns(mgd, tablename)) {
+		
+			g_warning("Failed to create metadata columns"); 
+			return FALSE;
+		}
+	}
+
+
 	return TRUE;
 }
 
 gboolean midgard_core_query_create_class_storage(
-		MidgardConnection *mgd, MidgardObjectClass *klass)
+		MidgardConnection *mgd, MidgardDBObjectClass *klass)
 {
 	g_return_val_if_fail(mgd != NULL, FALSE);
 	g_return_val_if_fail(klass != NULL, FALSE);
@@ -1682,7 +2104,15 @@ gboolean midgard_core_query_create_class_storage(
 		midgard_core_class_get_table(MIDGARD_DBOBJECT_CLASS(klass));
 	const gchar *classname = 
 		G_OBJECT_CLASS_NAME(G_OBJECT_CLASS(klass));
-	guint n_prop, i = 0;
+
+	/* Class with empty storage, ignore as silently as possible */
+	if (!tablename) {
+		g_debug("Can not create %s class table. None defined.", classname);
+		return TRUE;
+	}
+
+	guint n_prop;
+	guint i = 0;
 
 	if(__table_exists(mgd, tablename)) {
 
@@ -1708,8 +2138,10 @@ gboolean midgard_core_query_create_class_storage(
 	/* Get PRIMARY_KEY */
 	const gchar *pprop = NULL;
 	const gchar *pfield = NULL;
-	pprop = midgard_object_class_get_primary_property(klass);
 	const gchar *primary_field = NULL;
+	if (MIDGARD_IS_OBJECT_CLASS (klass))
+		pprop = midgard_object_class_get_primary_property(MIDGARD_OBJECT_CLASS(klass));
+	
 	if(pprop) {
 
 		pfield = midgard_core_class_get_property_colname(
@@ -1734,13 +2166,15 @@ gboolean midgard_core_query_create_class_storage(
 	}
 
 	g_strfreev(spltd);
-	i = 0;
 
-	if(!midgard_core_query_create_metadata_columns(mgd, tablename)) {
+	if (klass->dbpriv->has_metadata) {
+	
+		if(!midgard_core_query_create_metadata_columns(mgd, tablename)) {
 		
-		g_free(pspecs);
-		g_warning("Failed to create metadata columns"); 
-		return FALSE;
+			g_free(pspecs);
+			g_warning("Failed to create metadata columns"); 
+			return FALSE;
+		}
 	}
 	
 	GValue pval = {0, };
@@ -1753,12 +2187,12 @@ gboolean midgard_core_query_create_class_storage(
 	for (i = 0; i < n_prop; i++) {
 	
 		nick = midgard_core_class_get_property_tablefield
-			(MIDGARD_DBOBJECT_CLASS(klass), pspecs[i]->name);
-	
+			(MIDGARD_DBOBJECT_CLASS(klass), pspecs[i]->name);	
+
 		if (nick == NULL)
 			continue;
 
-		if (g_str_equal(pprop, pspecs[i]->name))
+		if ( pprop && (g_str_equal(pprop, pspecs[i]->name)))
 			continue;
 		
 		g_value_init(&pval, pspecs[i]->value_type);
@@ -1766,18 +2200,10 @@ gboolean midgard_core_query_create_class_storage(
 		
 		spltd = g_strsplit(nick, ".", 0);
 		
-		/* ML class */
-		if(midgard_object_class_is_multilang(klass)) {
-			mdc->column_name = spltd[1];
-			mdc->table_name = spltd[0];
-		} else {
-			mdc->column_name = spltd[1];
-			mdc->table_name = tablename;
-		}
-
-		mgd_type = midgard_reflection_property_get_midgard_type(
-				mrp, pspecs[i]->name);
+		mdc->column_name = spltd[1];
+		mdc->table_name = tablename;
 
+		mgd_type = midgard_reflection_property_get_midgard_type (mrp, pspecs[i]->name);
 		mdc->gtype = mgd_type;
 
 		switch(pspecs[i]->value_type) {
@@ -1806,7 +2232,7 @@ gboolean midgard_core_query_create_class_storage(
 
 			case G_TYPE_UINT:
 			case G_TYPE_INT:
-				if(g_str_equal(pfield, pspecs[i]->name)) {
+				if(pfield && (g_str_equal(pfield, pspecs[i]->name))) {
 					mdc->dbtype = "integer";
 					mdc->primary = TRUE;
 					mdc->autoinc = TRUE;
@@ -1855,28 +2281,36 @@ gboolean midgard_core_query_create_class_storage(
 	g_object_unref(mrp);
 	g_free(pspecs);
 
-	/* Create class specific columns */
-
-	/* sitegroup column in multilingual table */
-	if (midgard_object_class_is_multilang(klass)) {
-
-		mdc = midgard_core_dbcolumn_new();
-		mdc->column_name = "sitegroup";
-		mdc->table_name = 
-			midgard_core_class_get_property_table(MIDGARD_DBOBJECT_CLASS(klass), "lang");
-		mdc->dvalue = "0";
-		mdc->dbtype = "integer";
-		mdc->index = TRUE;
-		
-		midgard_core_query_add_column(mgd, mdc);
-		midgard_core_query_add_index(mgd, mdc);
-
-		g_free(mdc);
-	}
-
-	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, 
-			"%s - Table create: OK", 
-			g_type_name(G_OBJECT_CLASS_TYPE(klass)));
+	mgd_info ("%s - Table create: OK", g_type_name(G_OBJECT_CLASS_TYPE(klass)));
 
 	return TRUE;
 }
+
+MidgardDBJoin*
+midgard_core_dbjoin_new(void)
+{
+	MidgardDBJoin *mdbj = g_new(MidgardDBJoin, 1);
+
+	mdbj->type = NULL;
+	mdbj->typeid = 0;
+	mdbj->klass = NULL;
+
+	/* pointers references */
+	mdbj->left = NULL;
+	mdbj->right = NULL;
+
+	return mdbj;
+}
+
+void midgard_core_dbjoin_free (MidgardDBJoin *mdbj)
+{
+	g_assert(mdbj != NULL);
+
+	g_free(mdbj->type);
+	mdbj->type = NULL;
+
+	mdbj->typeid = 0;	
+
+	g_free(mdbj);
+}
+
