@@ -55,6 +55,9 @@ MidgardQueryBuilderPrivate *midgard_query_builder_private_new(void)
 	mqbp->grouping_ref = 0;
 	mqbp->group_constraint = NULL;
 	mqbp->joins = NULL;
+	mqbp->select_distinct = FALSE;
+	mqbp->param_join_exists = FALSE;
+	mqbp->blob_join_exists = FALSE;
 
 	return mqbp;
 }
@@ -142,26 +145,6 @@ static void _add_table_foreach(gpointer key, gpointer value, gpointer userdata)
 	*list = g_slist_append(*list, key);
 }
 
-gboolean midgard_core_qb_is_multilingual(MidgardQueryBuilder *builder)
-{
-	MidgardObjectClass *klass = (MidgardObjectClass*)g_type_class_peek(builder->priv->type);
-	MidgardConnection *mgd = builder->priv->mgd;
-	
-	if(!midgard_object_class_is_multilang(klass))
-		return FALSE;
-	
-	if(builder->priv->unset_lang)
-		return FALSE;
-	
-	if(MGD_GET_LANG_ID(mgd) == MGD_GET_DEFAULT_LANG_ID(mgd))
-		return FALSE;
-	
-	if(builder->priv->lang < 0)
-		return TRUE;
-	
-	return FALSE;
-}
-
 static void __sql_add_constraints(GString *sql, MidgardQueryBuilder *builder)
 {
 	GSList *clist = NULL;
@@ -196,33 +179,34 @@ static void __sql_add_constraints(GString *sql, MidgardQueryBuilder *builder)
 gchar *midgard_core_qb_get_sql(
 		MidgardQueryBuilder *builder, guint mode, gchar *select)
 {
-	g_assert(builder);
-        g_assert(builder->priv->schema->table);    
+	g_assert (builder != NULL);
+        g_assert (builder->priv->schema->table);    
 
         /* FIXME, error reporting needed */
-        if(builder->priv->error != 0) {
+        if (builder->priv->error != 0) {
 		g_warning("Can not create query");
 		return NULL;
-        }
-
-	guint multilang_fallback = midgard_core_qb_is_multilingual(builder);
-	MidgardConnection *mgd = builder->priv->mgd;
+        }	
 
 	/* SELECT */
-        GString *sql = g_string_new("SELECT ");
-	g_string_append(sql, select);
-	g_free(select);
-        
-	/* FROM */
-        g_string_append(sql, " FROM ");
-	GSList *tlist = NULL;
-	g_hash_table_foreach(builder->priv->tables, _add_table_foreach, &tlist);
+        GString *sql = g_string_new ("SELECT ");
+	
+	if (builder->priv->select_distinct)
+		g_string_append (sql, "DISTINCT ");
+	
+	g_string_append (sql, select);
+	g_free (select);
 
-	GString *tables = g_string_new("");
+	/* FROM */
+        g_string_append (sql, " FROM ");
+	GSList *tlist = NULL;
+	g_hash_table_foreach (builder->priv->tables, _add_table_foreach, &tlist);
+
+	GString *tables = g_string_new ("");
 
 	guint l = 0;
 	GSList *nlist; 
-	for(nlist = tlist; nlist != NULL; nlist = nlist->next) {
+	for (nlist = tlist; nlist != NULL; nlist = nlist->next) {
 	
 		if(l < 1)
 			g_string_append(tables, (gchar *) nlist->data);
@@ -241,33 +225,6 @@ gchar *midgard_core_qb_get_sql(
 	
 	__sql_add_constraints(sql, builder);
 
-	/* MULTILANG FALLBACK */
-	if(multilang_fallback) {
-	
-		guint lang, dlang;
-		lang = midgard_connection_get_lang_id(builder->priv->mgd, 
-				midgard_connection_get_lang(builder->priv->mgd));
-		dlang = midgard_connection_get_lang_id(builder->priv->mgd, 
-				midgard_connection_get_default_lang(builder->priv->mgd));
-
-
-		g_string_append_printf(sql, " AND %s_i.id IN (", builder->priv->schema->table);
-		g_string_append_printf(sql, " SELECT ABS(MIN(i_id)) as i_id FROM (SELECT %s_i.sid, "
-				"CASE WHEN %s_i.lang=%d THEN %s_i.id*-1 ELSE %s_i.id END AS i_id FROM "
-				"%s  WHERE %s_i.sid = %s.id AND %s_i.lang IN (%d,%d) ",
-				builder->priv->schema->table, builder->priv->schema->table,
-				lang,
-				builder->priv->schema->table, builder->priv->schema->table,
-				tables->str, builder->priv->schema->table, builder->priv->schema->table,
-				builder->priv->schema->table,
-				lang, dlang);
-		
-		g_string_append(sql, " AND ");
-		__sql_add_constraints(sql, builder);
-		
-		g_string_append(sql, " ) AS midgard_multilang_fallback_table GROUP BY sid )");
-	}
-
 	g_string_free(tables, TRUE);
 
 	/* OR&AND groupings */
@@ -299,7 +256,6 @@ gchar *midgard_core_qb_get_sql(
 	/* Join tables from ORDER BY */
 	GSList *olist = NULL;
 	GSList *jlist = NULL;
-	i = 0;
 	if(builder->priv->orders != NULL) {
 
 		for(olist = builder->priv->orders;
@@ -312,7 +268,6 @@ gchar *midgard_core_qb_get_sql(
 				g_string_append(sql,
 						((MidgardQueryConstraintPrivate*)jlist->data)->condition);                
 			}
-			i++;
 		}
 	}
 
@@ -326,50 +281,19 @@ gchar *midgard_core_qb_get_sql(
 			g_string_append(sql,
 					((MidgardQueryConstraintPrivate*)jlist->data)->condition);                
 		}
-		i++;
 	}
 
+	MidgardDBObjectClass *dbklass = g_type_class_peek(builder->priv->type);
 
-        if (builder->priv->schema->use_lang == 1) {
-
-                g_string_append(sql, " AND ");
-                g_string_append(sql, builder->priv->schema->table);
-                g_string_append(sql, ".id=");
-                g_string_append(sql, builder->priv->schema->table);
-                g_string_append(sql, "_i.sid");  
-               
-		if(!builder->priv->unset_lang
-				&& ( builder->priv->lang >= 0
-					|| (MGD_GET_LANG_ID(mgd) == MGD_GET_DEFAULT_LANG_ID(mgd)))){
-			
-			g_string_append(sql, " AND ");
-			g_string_append(sql, builder->priv->schema->table);
-			g_string_append_printf(sql,
-					"_i.lang = %d ",
-					builder->priv->lang > -1 ? builder->priv->lang : MGD_GET_LANG_ID(builder->priv->mgd));                  
-		}       
-        }           
-
-	MidgardUser *user = NULL;
-
-	if(builder->priv->mgd->priv->user != NULL) 
-		user = MIDGARD_USER(builder->priv->mgd->priv->user);
+	if (dbklass->dbpriv->has_metadata) {
 	
-	if(builder->priv->sitegrouped 
-			&& ((!user) || (user && !midgard_user_is_root(user)))) {
-
-                g_string_append(sql, " AND ");
-                g_string_append(sql, builder->priv->schema->table);
-                g_string_append_printf(
-                        sql, ".sitegroup IN (0, %u)",
-			midgard_connection_get_sitegroup_id(builder->priv->mgd));
-        }
-
-	if(!builder->priv->include_deleted) {
-		g_string_append(sql, " AND ");
-		g_string_append_printf(sql,
-				" %s.metadata_deleted = 0",
-				builder->priv->schema->table);
+		if(!builder->priv->include_deleted) {
+	
+			g_string_append(sql, " AND ");
+			g_string_append_printf(sql,
+					" %s.metadata_deleted = 0",
+					builder->priv->schema->table);
+		}
 	}
 
 	/* ORDER BY */
@@ -470,7 +394,6 @@ static gboolean __set_schema_property_attr(
 	MgdSchemaPropertyAttr *attr = NULL;
 	const gchar *target_property = NULL;
 	const gchar *link_property = NULL;
-	gboolean is_link = FALSE;
 
 	/*
 	GParamSpec *pspec = 
@@ -497,8 +420,12 @@ static gboolean __set_schema_property_attr(
 	} 
 	
 	if(g_str_equal(_RESERVED_BLOB_NAME, name)) {
-	
-		__join_reference(builder, (const gchar *)(*constraint)->priv->current->table, _RESERVED_BLOB_TABLE);
+
+		if (builder && !builder->priv->blob_join_exists) {
+
+			__join_reference(builder, (const gchar *)(*constraint)->priv->current->table, _RESERVED_BLOB_TABLE);
+			builder->priv->blob_join_exists = TRUE;
+		}
 
 		*klass = 
 			g_type_class_peek(g_type_from_name("midgard_attachment"));
@@ -508,12 +435,23 @@ static gboolean __set_schema_property_attr(
 	
 	} else 	if(g_str_equal(_RESERVED_PARAM_NAME, name)) {
 		
-		__join_reference(builder, (const gchar *)(*constraint)->priv->current->table, _RESERVED_PARAM_TABLE);
+		if (builder && !builder->priv->param_join_exists) {
+
+			__join_reference(builder, (const gchar *)(*constraint)->priv->current->table, _RESERVED_PARAM_TABLE);
+			builder->priv->param_join_exists = TRUE;
+		}
 
 		*klass = 
 			g_type_class_peek(g_type_from_name("midgard_parameter"));
 		(*constraint)->priv->current->table = _RESERVED_PARAM_TABLE;
 
+		/* Select DISTINCT.
+		 * Constraints might be grouped with "OR" operator and duplicated 
+		 * records can be returned explicitly */
+		if (builder) {
+			builder->priv->select_distinct = TRUE;
+		}
+
 		return TRUE;
 	
 	/* User defined properties */
@@ -540,7 +478,6 @@ static gboolean __set_schema_property_attr(
 
 		target_property = NULL;
 		link_property = NULL;
-		is_link = FALSE;
 
 		MidgardReflectionProperty *mrp =
 			midgard_reflection_property_new(MIDGARD_DBOBJECT_CLASS(*klass));
@@ -588,11 +525,15 @@ static gboolean __set_schema_property_attr(
 
 				mqcp->condition = g_string_free(cond, FALSE);
 
-				midgard_core_qb_add_table(builder, mqcp->prop_left->table);
-				midgard_core_qb_add_table(builder, mqcp->prop_right->table);
+				if (builder) {
+					midgard_core_qb_add_table(builder, mqcp->prop_left->table);
+					midgard_core_qb_add_table(builder, mqcp->prop_right->table);
+				}
 
-				if(!__add_join(&builder->priv->joins, mqcp)) {
-					midgard_query_constraint_private_free(mqcp);
+				if (builder) {
+					if(!__add_join(&builder->priv->joins, mqcp)) {
+						midgard_query_constraint_private_free(mqcp);
+					}
 				}
 			}
 
@@ -609,14 +550,7 @@ static gboolean __set_schema_property_attr(
 
 	const gchar *table, *field;
 
-	/* FIXME, it seems to be ported from 1.9 without dbobject usage */
-	/* Fixed in Midgard2 already */
-	if(g_str_equal(name, "sitegroup")) {
-		
-		table = midgard_core_class_get_table(MIDGARD_DBOBJECT_CLASS(*klass)); 
-		field = "sitegroup";
-	
-	} else if(g_str_equal(name, "guid")) {
+	if (g_str_equal(name, "guid")) {
 		
 		table = midgard_core_class_get_table(MIDGARD_DBOBJECT_CLASS(*klass));
 		field = "guid";
@@ -635,24 +569,23 @@ static gboolean __set_schema_property_attr(
 	return TRUE;
 }
 
-gboolean midgard_core_qb_parse_property(
-		MidgardQueryBuilder *builder, 
-		MidgardQueryConstraint **constraint, 
-		const gchar *name)
+gboolean 
+midgard_query_constraint_parse_property (MidgardQueryConstraint **constraint, MidgardDBObjectClass *klass, const gchar *name)
 {
-	g_assert(builder != NULL);
 	g_assert(name != NULL);
 
-	MidgardDBObjectClass *klass =
-		MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type));
+	if (!klass)
+		klass = MIDGARD_DBOBJECT_CLASS((*constraint)->priv->klass);
+
 	(*constraint)->priv->current->table = midgard_core_class_get_table(MIDGARD_DBOBJECT_CLASS(klass)); 
 
+	/* This is optional and might be set to NULL explicitly.
+	   We need to parse property for any class which needs this */
+	MidgardQueryBuilder *builder = (*constraint)->priv->builder;
+
 	gchar **spltd = g_strsplit(name, ".", 0);
 	guint i = 0;
 
-	if((*constraint)->priv->builder == NULL)
-		(*constraint)->priv->builder = builder;
-
 	/* We can support max 3 tokens */
 	while(spltd[i] != NULL)
 		i++;
@@ -680,13 +613,13 @@ gboolean midgard_core_qb_parse_property(
 			midgard_core_class_get_property_colname(
 					MIDGARD_DBOBJECT_CLASS(klass), name);
 		(*constraint)->priv->current->table = stable;
-		midgard_core_qb_add_table(builder, stable);
+		
+		if (builder)
+			midgard_core_qb_add_table(builder, stable);
+
 		(*constraint)->priv->klass = G_OBJECT_CLASS(klass);
 		(*constraint)->priv->propname = name;
-		
-		if(g_str_equal(name, "sitegroup")) 
-			(*constraint)->priv->current->field = "sitegroup";
-			
+					
 		if(g_str_equal(name, "guid")) 
 			(*constraint)->priv->current->field = "guid";
 
