@@ -34,22 +34,20 @@ static void _action_create_callback(MgdObject *object, gpointer ud)
 {
 	MidgardMetadata *mdata = (MidgardMetadata *) ud;
 
-	gchar *person_guid = NULL;
+	gchar *person_guid = "";
 	MidgardConnection *mgd = MGD_OBJECT_CNC(object);
 	MidgardUser *user = midgard_connection_get_user(mgd);
-	MgdObject *person = NULL;
-	
-	if (user)
-		person = midgard_user_get_person(user);
 
-	if(person){
-		if(G_IS_OBJECT(G_OBJECT(person)))
-			person_guid = (gchar *)MGD_OBJECT_GUID(person);			
-	} else {
-		/* g_info("Anonymous mode"); */
-		person_guid = "";
+	if (user) {
+		const MgdObject *person = midgard_user_get_person(user);
+
+		if (person) {
+			if (G_IS_OBJECT(G_OBJECT(person)))
+				person_guid = (gchar *)MGD_OBJECT_GUID(person);			
+		}
 	}
 
+
 	GValue *tval = midgard_timestamp_new_current();
 	
 	/* set creator */
@@ -68,26 +66,24 @@ static void _action_create_callback(MgdObject *object, gpointer ud)
 	midgard_core_metadata_set_revisor(mdata, person_guid);
 	
 	g_value_unset(tval);
+	g_free (tval); 
 }
 
 static void _action_update_callback(MgdObject *object, gpointer ud)
 {
 	MidgardMetadata *mdata = (MidgardMetadata *) ud;
 	
-	gchar *person_guid = NULL;
+	gchar *person_guid = "";
 	MidgardConnection *mgd = MGD_OBJECT_CNC(object);
 	MidgardUser *user = midgard_connection_get_user(mgd);
-	MgdObject *person = NULL;
-	
-	if (user)
-		person = midgard_user_get_person(user);
 
-	if(person){
-		if(G_IS_OBJECT(G_OBJECT(person)))
-			person_guid = (gchar *)MGD_OBJECT_GUID(person);			
-	} else {
-		/* g_info("Anonymous mode"); */
-		person_guid = "";
+	if (user) {
+		const MgdObject *person = midgard_user_get_person(user);
+
+		if (person) {
+			if (G_IS_OBJECT(G_OBJECT(person)))
+				person_guid = (gchar *)MGD_OBJECT_GUID(person);			
+		}
 	}
 
 	GValue *tval = midgard_timestamp_new_current();
@@ -100,8 +96,9 @@ static void _action_update_callback(MgdObject *object, gpointer ud)
 
 	/* set revision */
 	midgard_core_metadata_increase_revision(mdata);
-			
+	
 	g_value_unset(tval);	
+	g_free (tval);
 }
 
 static void _action_delete_callback(MgdObject *object, gpointer ud)
@@ -112,74 +109,42 @@ static void _action_delete_callback(MgdObject *object, gpointer ud)
 static void _action_import_callback(MgdObject *object, gpointer ud)
 {
 	MidgardMetadata *mdata = (MidgardMetadata *) ud;	
-
-	gchar *person_guid = NULL;
-	MidgardConnection *mgd = MGD_OBJECT_CNC(object);
-	MidgardUser *user = midgard_connection_get_user(mgd);
-	MgdObject *person = NULL;
-	
-	if (user)
-		person = midgard_user_get_person(user);
-
-	if(person){
-		if(G_IS_OBJECT(G_OBJECT(person)))
-			person_guid = (gchar *)MGD_OBJECT_GUID(person);			
-	} else {
-		/* g_info("Anonymous mode"); */
-		person_guid = "";
-	}
-
 	GValue *tval = midgard_timestamp_new_current();
+
 	/* set imported */
-	midgard_core_metadata_set_imported(mdata, tval);
+	midgard_core_metadata_set_imported (mdata, tval);
 	
-	g_value_unset(tval);
+	g_value_unset (tval);
+	g_free (tval);
 }
 
 static void _action_export_callback(MgdObject *object, gpointer ud)
 {
 	MidgardMetadata *mdata = (MidgardMetadata *) ud;	
-
-	gchar *person_guid = NULL;
-	MidgardConnection *mgd = MGD_OBJECT_CNC(object);
-	MidgardUser *user = midgard_connection_get_user(mgd);
-	MgdObject *person = NULL;
-	
-	if (user)
-		person = midgard_user_get_person(user);
-
-	if(person){
-		if(G_IS_OBJECT(G_OBJECT(person)))
-			person_guid = (gchar *)MGD_OBJECT_GUID(person);			
-	} else {
-		/* g_info("Anonymous mode"); */
-		person_guid = "";
-	}
-
 	const GValue *tval = (GValue *)midgard_timestamp_new_current();
 
 	/* This is backward "compatible" defaults set.
 	 * It's done in case when some old object has been never
 	 * touched by new API */
-	if(!mdata->priv->revised
-			&& !mdata->priv->created){
+	if (!mdata->priv->revised && !mdata->priv->created) {
 		midgard_core_metadata_set_created(mdata, tval);
 		midgard_core_metadata_set_revised(mdata, tval);
 	}
-	if(!mdata->priv->revised
-			&& mdata->priv->created){
+
+	if (!mdata->priv->revised && mdata->priv->created) {
 		midgard_core_metadata_set_created(mdata, tval);
 		midgard_core_metadata_increase_revision(mdata);
 	}
-	if(!mdata->priv->created
-			&& mdata->priv->revised){
+
+	if (!mdata->priv->created && mdata->priv->revised) {
 		midgard_core_metadata_set_created(mdata, tval);
 	}
 		
 	/* set exported */
 	midgard_core_metadata_set_exported(mdata, tval);
 
-	g_value_unset((GValue *)tval);
+	g_value_unset ((GValue *)tval);
+	g_free ((GValue *)tval);
 }
 
 /**
@@ -412,6 +377,27 @@ _metadata_get_property (GObject *object, guint property_id,
 	}
 }
 
+static GObjectClass *__parent_class = NULL;
+
+static GObject *
+_metadata_constructor (GType type, guint n_construct_properties,
+			GObjectConstructParam *construct_properties)
+{
+
+	GObject *object = (GObject *)
+		G_OBJECT_CLASS (__parent_class)->constructor (type,
+				n_construct_properties,
+				construct_properties);
+
+	return G_OBJECT(object);
+}
+
+static void
+_metadata_dispose (GObject *object)
+{
+	__parent_class->dispose (object);
+}
+
 static void 
 _metadata_instance_init(GTypeInstance *instance, gpointer g_class) 
 {	
@@ -446,21 +432,13 @@ _metadata_instance_init(GTypeInstance *instance, gpointer g_class)
 
 	self->priv->approve_is_set = FALSE;
 	self->priv->is_approved = FALSE;
-
-	MidgardMetadataClass *klass = MIDGARD_METADATA_CLASS(g_class);
-
-	self->dbpriv = g_new(MidgardDBObjectPrivate, 1);
-	self->dbpriv->storage_data = klass->dbpriv->storage_data;
-	self->dbpriv->guid = NULL;
-	self->dbpriv->sg = -1;
-	self->dbpriv->mgd = NULL;
 }
 
 static void 
 _metadata_object_finalize(GObject *object) 
 {
 	g_assert(object);
-	
+
 	MidgardMetadata *self = (MidgardMetadata *)object;
 
 	/* Free private data */
@@ -481,6 +459,8 @@ _metadata_object_finalize(GObject *object)
 	g_free(self->priv->imported);
 	
 	g_free(self->priv);
+
+	__parent_class->finalize (object);
 }
 
 /* Set properties when object is created */
@@ -520,14 +500,16 @@ void midgard_metadata_set_update(MgdObject *object)
 /* Initialize class */
 static void
 _metadata_class_init (gpointer g_class, gpointer g_class_data)
-{
-	
+{	
 	GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
 	MidgardMetadataClass *klass = MIDGARD_METADATA_CLASS (g_class); 
 	GParamSpec *pspec;
-	
+	__parent_class = g_type_class_peek_parent (g_class);
+
+	gobject_class->constructor = _metadata_constructor;	
 	gobject_class->set_property = _metadata_set_property;
 	gobject_class->get_property = _metadata_get_property;
+	gobject_class->dispose = _metadata_dispose;
 	gobject_class->finalize = _metadata_object_finalize;
 
 	/* Register signals */
@@ -1025,7 +1007,7 @@ gchar *__get_node_content_string(xmlNode *node)
 	return (gchar*)decoded;
 }
 
-void __set_date_from_node(xmlNode *node,  MidgardTimestamp *date)
+void __set_date_from_node(xmlNode *node,  MidgardTimestamp **date)
 {
 	if(!node)
 		return;
@@ -1038,10 +1020,10 @@ void __set_date_from_node(xmlNode *node,  MidgardTimestamp *date)
 		return;
 	}
 
-	if (date != NULL)
-		g_free(date);
+	if (*date != NULL)
+		g_free(*date);
 
-	date = midgard_timestamp_new_from_iso8601(content);
+	*date = midgard_timestamp_new_from_iso8601(content);
 	
 	return;
 }
@@ -1111,13 +1093,13 @@ void __set_from_xml_node(MidgardDBObject *object, xmlNode *node)
 	self->priv->creator = __get_node_content_string(lnode);
 
 	lnode = __metadata_lookup_node(node, "created");
-	__set_date_from_node(lnode, self->priv->created);	
+	__set_date_from_node(lnode, &self->priv->created);	
 
 	lnode = __metadata_lookup_node(node, "revisor");
 	self->priv->revisor = __get_node_content_string(lnode);
 	
 	lnode = __metadata_lookup_node(node, "revised");
-	__set_date_from_node(lnode, self->priv->revised);	
+	__set_date_from_node(lnode, &self->priv->revised);	
 
 	lnode = __metadata_lookup_node(node, "revision");
 	self->priv->revision = __get_node_content_uint(lnode);
@@ -1126,13 +1108,13 @@ void __set_from_xml_node(MidgardDBObject *object, xmlNode *node)
 	self->priv->locker = __get_node_content_string(lnode);
 	
 	lnode = __metadata_lookup_node(node, "locked");
-	__set_date_from_node(lnode, self->priv->locked);	
+	__set_date_from_node(lnode, &self->priv->locked);	
 
 	lnode = __metadata_lookup_node(node, "approver");
 	self->priv->approver = __get_node_content_string(lnode);
 	
 	lnode = __metadata_lookup_node(node, "approved");
-	__set_date_from_node(lnode, self->priv->approved);	
+	__set_date_from_node(lnode, &self->priv->approved);	
 
 	lnode = __metadata_lookup_node(node, "authors");
 	self->priv->authors = __get_node_content_string(lnode);
@@ -1141,10 +1123,10 @@ void __set_from_xml_node(MidgardDBObject *object, xmlNode *node)
 	self->priv->owner = __get_node_content_string(lnode);
 
 	lnode = __metadata_lookup_node(node, "schedulestart");
-	__set_date_from_node(lnode, self->priv->schedule_start);	
+	__set_date_from_node(lnode, &self->priv->schedule_start);	
 
 	lnode = __metadata_lookup_node(node, "scheduleend");
-	__set_date_from_node(lnode, self->priv->schedule_end);	
+	__set_date_from_node(lnode, &self->priv->schedule_end);	
 
 	lnode = __metadata_lookup_node(node, "hidden");
 	self->priv->hidden = __get_node_content_bool(lnode);
@@ -1156,16 +1138,16 @@ void __set_from_xml_node(MidgardDBObject *object, xmlNode *node)
 	self->priv->size = __get_node_content_uint(lnode);
 
 	lnode = __metadata_lookup_node(node, "published");
-	__set_date_from_node(lnode, self->priv->published);	
+	__set_date_from_node(lnode, &self->priv->published);	
 
 	lnode = __metadata_lookup_node(node, "score");
 	self->priv->score = __get_node_content_int(lnode);
 	
 	lnode = __metadata_lookup_node(node, "imported");
-	__set_date_from_node(lnode, self->priv->imported);	
+	__set_date_from_node(lnode, &self->priv->imported);	
 
 	lnode = __metadata_lookup_node(node, "exported");
-	__set_date_from_node(lnode, self->priv->exported);	
+	__set_date_from_node(lnode, &self->priv->exported);	
 
 	lnode = __metadata_lookup_node(node, "deleted");
 	self->priv->deleted = __get_node_content_bool(lnode);
