@@ -160,18 +160,15 @@ const GValue *midgard_object_get_parameter(MgdObject *self,
  * @domain: parameter's domain string
  * @name: parameter's name string
  * @value: a GValue value which should be set for domain&name pair
- * @lang: boolean which determines if default language should be forced
  *
  * Creates object's parameter object if it doesn't exists, updates otherwise.
  *
  * Returns: %TRUE on success, %FALSE otherwise
  */ 
-gboolean midgard_object_set_parameter(MgdObject *self,
-		const gchar *domain, const gchar *name,
-		GValue *value, gboolean lang) 
+gboolean 
+midgard_object_set_parameter (MgdObject *self, const gchar *domain, const gchar *name, GValue *value) 
 {
-
-	g_assert(self != NULL);
+	g_return_val_if_fail (self != NULL, FALSE);
 
 	if(!__is_guid_valid(self))
 		return FALSE;
@@ -214,10 +211,7 @@ gboolean midgard_object_set_parameter(MgdObject *self,
 				"name", name,
 				"parentguid", self->dbpriv->guid,
 				NULL);
-		
-		/* Set parameter's sitegroup using "parent's" sitegroup */
-		param->dbpriv->sg = self->dbpriv->sg;
-
+			
 		g_object_set_property(G_OBJECT(param), "value", value);
 
 		if(midgard_object_create(param)) {
@@ -356,45 +350,6 @@ MgdObject **midgard_object_list_parameters(MgdObject *self, const gchar *domain)
 	return objects;
 }
 
-/**
- * midgard_object_parameter_create:
- * @self: #MgdObject instance
- * @n_params: number of properties
- * @parameters: properties list
- *
- * Creates object's parameter using given properties.
- *
- * Returns: newly created #MgdObject of midgard_parameter class or %NULL on failure
- */
-MgdObject *midgard_object_parameter_create(MgdObject *self, 
-		guint n_params, const GParameter *parameters)
-{
-	return NULL;
-
-	g_return_val_if_fail(self != NULL, NULL);
-	
-	if(n_params < 3 || !parameters) {
-		
-		g_warning("3 parameters expected at least ( domain, name, value )");
-		return NULL;
-	}
-	
-	guint i;	
-	const gchar *prop_name;
-	GValue pval;
-
-	for( i = 0; i < n_params; i++) {
-		
-		prop_name = parameters[i].name;
-		pval = parameters[i].value;
-
-		if(g_str_equal(prop_name, "domain")) {
-
-		}
-
-	}
-}
-
 /**
  * midgard_object_delete_parameters:
  * @self: #MgdObject instance
