@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2005 Jukka Zitting <jz@yukatan.fi>
- * Copyright (c) 2005,2007 Piotr Pokora <piotr.pokora@infoglob.pl>
+ * Copyright (c) 2005, 2007, 2009 Piotr Pokora <piotrek.pokora@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -33,16 +33,31 @@
 #include "midgard_reflection_property.h"
 #include "midgard_object.h"
 #include "midgard_metadata.h"
-#include "midgard_sitegroup.h"
 #include "midgard_user.h"
 #include "midgard_core_object_class.h"
 #include "midgard_dbobject.h"
 #include "midgard_timestamp.h"
 #include "midgard_core_metadata.h"
+#include "midgard_view.h"
 
 /* Internal prototypes , I am not sure if should be included in API */
 gchar *midgard_query_builder_get_object_select(MidgardQueryBuilder *builder, guint select_type);
 
+/*
+static gboolean _mqb_is_grouping(MidgardQueryBuilder *builder)
+{
+	return FALSE;
+
+	if(builder->priv->is_grouping) {
+		
+		g_warning("Group condition not correctly closed ( missed 'end_group' ?) ");
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+*/
+
 static gboolean __type_is_valid(GType type)
 {
 	GType _type = type;
@@ -57,19 +72,28 @@ static gboolean __type_is_valid(GType type)
 	if(g_type_parent(_type) != MIDGARD_TYPE_DBOBJECT) {
 		
 		/* if fails, try MIDGARD_TYPE_DBOBJECT */
-		if(g_type_parent(_type) != MIDGARD_TYPE_OBJECT) {
+		if(g_type_parent(_type) == MIDGARD_TYPE_OBJECT
+				|| g_type_parent(_type) == MIDGARD_TYPE_VIEW) {
 			
-			return FALSE;
+			return TRUE;
 		
 		} else {
 
-			return TRUE;
+			return FALSE;
 		}
 	}
 
 	return TRUE;
 }
 
+/**
+ * midgard_query_builder_new():
+ * @mgd: #MidgardConnection instance
+ * @classname: any #MidgardDBObjectClass derived class' name
+ *
+ * Returns: new #MidgardQueryBuilder instance or %NULL if target class is not registered in GType system 
+ * or it's not #MidgardDBObjectClass class derived one.
+ */
 MidgardQueryBuilder *midgard_query_builder_new(
         MidgardConnection *mgd, const gchar *classname)
 {
@@ -92,8 +116,6 @@ MidgardQueryBuilder *midgard_query_builder_new(
 
         builder->priv->mgd = mgd;
         builder->priv->type = g_type_from_name(classname);
-        builder->priv->lang = -1;
-	builder->priv->unset_lang = FALSE;
 	builder->priv->include_deleted = FALSE;
         builder->priv->error = 0;	
 	
@@ -120,7 +142,6 @@ MidgardQueryBuilder *midgard_query_builder_new(
 
         builder->priv->offset = 0;
         builder->priv->limit = G_MAXUINT;
-	builder->priv->sitegrouped = TRUE;
 
         if (builder->priv->type && builder->priv->schema) {
                 return builder;
@@ -137,6 +158,88 @@ void midgard_query_builder_free(MidgardQueryBuilder *builder)
 	midgard_query_builder_private_free(builder->priv);
 }
 
+/**
+ * midgard_query_builder_add_constraint():
+ * @builder: #MidgardQueryBuilder instance
+ * @name: property name used for this constraint 
+ * @op: comparison operator
+ * @value: value used in comparison
+ *
+ * Adds a constraint to the given query builder. The constraint is
+ * expressed as a triple of a field name, a comparison operator, and
+ * a comparison value. 
+ * 
+ * <para>
+ * @name referes to a property of the queried Midgard object class. 
+ * For example: #MidgardQueryBuilder has been initialized for person 
+ * class which has lastname property registered.
+ * <example>
+ * <programlisting>
+ * 
+ * GValue value = {0, };
+ * g_value_init(&value, G_TYPE_STRING);
+ * g_value_set_string(&value, "smith");
+ *
+ * midgard_query_builder_add_constraint(builder, "lastname", "=", &value);
+ * 
+ * </programlisting>
+ * </example>
+ * </para>
+ * <para>
+ * It also can be name of the linked class property or reserved one. 
+ * A dot '.' is used to separate properties for special constraints.
+ * If such special constraint property is used, #MidgardQueryBuilder
+ * performs right join.
+ * <itemizedlist>
+ * <listitem><para>
+ * First property is the one registered for given class which is a link
+ * to property of different class. Second is a property of target class.
+ * For example: person object has property 'friend' which holds some identifier
+ * (id or guid) to friend class property, and friend class has property 'nick'.
+ * In such case we can use constraint and comparison using friend property,
+ * even if #MidgardQueryBuilder has been initialized for person class.
+ * <example>
+ * <programlisting>
+ * 
+ * GValue value = {0, };
+ * g_value_init(&value, G_TYPE_STRING);
+ * g_value_set_string(&value, "Lancelot");
+ *
+ * midgard_query_builder_add_constraint(builder, "friend.nick", "=", &value);
+ *
+ * </programlisting>
+ * </example>
+ * </para></listitem>
+ * <listitem><para>
+ * There are three reserved words which have special meaning for query builder.
+ * 'metadata', 'parameter' and 'attachment'. If one of them is used, query builder
+ * will make (if necessary) right join and query objects against dependent class table.
+ * <example>
+ * <programlisting>
+ * 
+ * GValue value = {0, };
+ * g_value_init(&value, G_TYPE_STRING);
+ * g_value_set_string(&value, "avatar");
+ *
+ * midgard_query_builder_add_constraint(builder, "attachment.name", "=", &value);
+ *
+ * </programlisting>
+ * </example>
+ * </para></listitem>
+ * </itemizedlist>
+ * </para>
+ *
+ * <para>
+ * The comparison operator is a string representation of the requested comparison. 
+ * Available operators are =, <>, <, >, <=, >=, LIKE, NOT LIKE, IN, INTREE. 
+ * </para>
+ * 
+ * <para>
+ * The given @value is copied and converted into the property type before comparison.
+ * </para>
+ *  
+ * Returns: %TRUE if constraint is valid, %FALSE otherwise
+ */
 gboolean midgard_query_builder_add_constraint(
         MidgardQueryBuilder *builder,
         const gchar *name, const gchar *op, const GValue *value) 
@@ -147,11 +250,13 @@ gboolean midgard_query_builder_add_constraint(
         g_assert(value);
      
 	MidgardQueryConstraint *constraint = midgard_query_constraint_new();
+	midgard_query_constraint_set_builder(constraint, builder);
+	midgard_query_constraint_set_class(constraint, MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type)));
 	
 	if(!midgard_query_constraint_add_operator(constraint, op))
 		return FALSE;
 		
-	if(!midgard_core_qb_parse_property(builder, &constraint, name))
+	if(!midgard_query_constraint_parse_property(&constraint, MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type)), name))
 		return FALSE;
 
 	if(!midgard_query_constraint_add_value(constraint, value))
@@ -169,16 +274,42 @@ gboolean midgard_query_builder_add_constraint(
 		midgard_group_constraint_add_constraint(group, constraint);
 		return TRUE;
 	
-	} else {
+	} 
 
-		midgard_core_qb_add_constraint(builder, constraint);
-	}
-
-	midgard_core_qb_add_constraint(builder, constraint);
+	midgard_core_qb_add_constraint(builder, constraint);	
 
 	return TRUE;
 }
 
+/**
+ * midgard_query_builder_add_constraint_with_property:
+ * @builder: #MidgardQueryBuilder instance
+ * @property_a: property name
+ * @op: comparison operator
+ * @property_b: property name
+ *
+ * Adds named property constraint to the given query builder.
+ * Unlike add_constraint method, this one accepts property name
+ * instead of scalar value. The difference is that with add_constraint
+ * method you can compare property with particular value, while using 
+ * add_constraint_with_property method you can compare two different 
+ * properties without any need to know their values. 
+ * For example, you should use this method if you want to select only 
+ * those objects which has been revised after publication time, and particular 
+ * date doesn't matter.
+ *
+ * <example>
+ * <programlisting>
+ * 
+ * midgard_query_builder_add_constraint_with_property(builder, "metadata.revised", ">", "metadata.published");
+ *
+ * </programlisting>
+ * </example>
+ *
+ * @See: midgard_query_builder_add_constraint()
+ *
+ * Returns: %TRUE if properties' names are valid, %FALSE otherwise
+ */ 
 gboolean midgard_query_builder_add_constraint_with_property(
 		MidgardQueryBuilder *builder, const gchar *property_a,
 		const gchar *op, const gchar *property_b)
@@ -188,12 +319,16 @@ gboolean midgard_query_builder_add_constraint_with_property(
 	g_assert(op != NULL);
 	g_assert(property_b != NULL);
 
-	MidgardQueryConstraint *constraint = midgard_query_constraint_new();
+	MidgardDBObjectClass *klass = MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type));
 
-	if(!midgard_core_qb_parse_property(builder, &constraint, property_a))
+	MidgardQueryConstraint *constraint = midgard_query_constraint_new();
+	midgard_query_constraint_set_builder(constraint, builder);
+	midgard_query_constraint_set_class(constraint, klass);
+
+	if(!midgard_query_constraint_parse_property(&constraint, klass, property_a))
 		return FALSE;
 	constraint->priv->current = constraint->priv->prop_right;
-	if(!midgard_core_qb_parse_property(builder, &constraint, property_b))
+	if(!midgard_query_constraint_parse_property(&constraint, klass, property_b))
 		return FALSE;
 
 	constraint->priv->condition_operator = g_strdup(op);
@@ -217,8 +352,22 @@ gboolean midgard_query_builder_add_constraint_with_property(
 	return TRUE;
 }
 
-gboolean midgard_query_builder_begin_group(
-                MidgardQueryBuilder *builder, const gchar *type) 
+/**
+ * midgard_query_builder_begin_group:
+ * @builder: #MidgardQueryBuilder instance
+ * @type: group type
+ *
+ * Starts a constraint group of the given type. A conjunctive constraint
+ * group @type (AND) requires that all component constraints match the
+ * queried objects, while a disjunctive group @type (OR) requires just
+ * one of the component constraints to match.
+ *
+ * Allowed group @type: 'AND' or 'OR'
+ *
+ * Returns: %TRUE if the @type is valid, %FALSE otherwise
+ */
+gboolean 
+midgard_query_builder_begin_group(MidgardQueryBuilder *builder, const gchar *type) 
 {
 	g_assert(builder != NULL);
 	
@@ -253,7 +402,20 @@ gboolean midgard_query_builder_begin_group(
 	return TRUE;
 }
 
-gboolean midgard_query_builder_end_group(MidgardQueryBuilder *builder) 
+/**
+ * midgard_query_builder_end_group:
+ * @builder: #MidgardQueryBuilder instance
+ * 
+ * Closes the most recently opened constraint group. The client should
+ * ensure proper nesting by closing all constraint groups before the
+ * containing query is executed.
+ *
+ * Returns: %TRUE if a constraint group was closed, or %FALSE if no
+ * open constraint groups were found
+ *
+ */
+gboolean 
+midgard_query_builder_end_group (MidgardQueryBuilder *builder) 
 {
 	g_assert(builder != NULL);
 
@@ -282,8 +444,27 @@ gboolean midgard_query_builder_end_group(MidgardQueryBuilder *builder)
 	return TRUE;
 }
 
-gboolean midgard_query_builder_add_order(
-        MidgardQueryBuilder *builder, const gchar *name, const gchar *dir)
+/**
+ * midgard_query_builder_add_order:
+ * @builder: #MidgardQueryBuilder instance
+ * @name: property name
+ * @dir: sort direction
+ *
+ * Adds an ordering constraint to the query. An ordering constraint consists
+ * of a property name and a sort direction. The objects returned by this
+ * query will be sorted by the given property in the given direction
+ * (ascending or descending). Multiple ordering constraints are applied in
+ * the order they were added.
+ *
+ * Allowed sort directions: 'ASC' or 'DESC'
+ * 
+ * Property name pattern is described in midgard_query_builder_add_constraint()
+ * 
+ * Returns: %TRUE if the ordering constraint is valid, %FALSE otherwise
+ *
+ */
+gboolean 
+midgard_query_builder_add_order (MidgardQueryBuilder *builder, const gchar *name, const gchar *dir)
 {
         g_assert(builder != NULL);
         g_assert(name != NULL);
@@ -303,99 +484,112 @@ gboolean midgard_query_builder_add_order(
 	}
 }
 
-void midgard_query_builder_set_limit(
-        MidgardQueryBuilder *builder, guint limit)
+/**
+ * midgard_query_builder_set_limit:
+ * @builder: #MidgardQueryBuilder instance
+ * @limit: query limit
+ * 
+ * Sets the maximum number of objects to return when the query is executed.
+ * A query will by default return all matching objects, but the @limit setting
+ * can be used to restrict the potentially large number of returned objects.
+ * The @limit is applied only after the matching objects have been identified
+ * and sorted and after the optional start offset has been applied.
+ *
+ * Setting a @limit on the number of returned objects is normally only
+ * reasonable when one or more ordering constraints and optionally an offset
+ * setting are applied to the query. 
+ *
+ * @See: midgard_query_builder_set_offset()
+ */
+void 
+midgard_query_builder_set_limit (MidgardQueryBuilder *builder, guint limit)
 {
 	g_assert(builder != NULL);
 
         builder->priv->limit = limit;
 }
 
-void midgard_query_builder_set_offset(
-        MidgardQueryBuilder *builder, guint offset)
+/**
+ * midgard_query_builder_set_offset:
+ * @builder: #MidgardQueryBuilder instance
+ * @offset: query offset
+ *
+ * Sets the start @offset of the objects to return when the query is executed.
+ * The start @offset is applied after all the matching objects have been
+ * identified and sorted according to the given ordering constraints. The
+ * first @offset objects are skipped and only the remaining (if any) objects
+ * are returned to the client.
+ *
+ * Setting a start offset is normally only reasonable when one or more
+ * ordering constraints are applied to the query. A start offset is usually
+ * accompanied by a limit setting.
+ * 
+ * @See: midgard_query_builder_set_offset()
+ */
+void 
+midgard_query_builder_set_offset (MidgardQueryBuilder *builder, guint offset)
 {
         g_assert(builder != NULL);
 
         builder->priv->offset = offset;
 }
 
-void midgard_query_builder_set_lang(MidgardQueryBuilder *builder, gint lang)
+static GList *midgard_query_builder_execute_or_count(MidgardQueryBuilder *builder, MidgardTypeHolder *holder, guint select_type)
 {
 	g_assert(builder != NULL);
-	
-	builder->priv->lang = lang;
-}
 
-void midgard_query_builder_unset_languages(MidgardQueryBuilder *builder)
-{
-	g_assert(builder);
-	
-	builder->priv->unset_lang = TRUE;
-}
-
-gint _compare_ml_guid(gconstpointer a, gconstpointer b){
-
-        if(g_ascii_strcasecmp(a,b) == 0) return 0;
-        return 1;
-}
-
-static GList *midgard_query_builder_execute_or_count(
-	MidgardQueryBuilder *builder, MidgardTypeHolder *holder, guint select_type)
-{
-        g_assert(builder != NULL);
-
-	if(builder->priv->grouping_ref > 0) {
-		
+	if (builder->priv->grouping_ref > 0) {
 		g_warning("Incorrect constraint grouping. Missed 'end_group'?");
 		return NULL;
 	}
 
 	MidgardUser *user = midgard_connection_get_user(builder->priv->mgd);
 
-	if(builder->priv->type == MIDGARD_TYPE_SITEGROUP) {
-		
-		if(!midgard_user_is_root(user)) {
-			
-			MIDGARD_ERRNO_SET(builder->priv->mgd, MGD_ERR_ACCESS_DENIED);
-			g_warning("Type incompatible with Midgard Query Builder");
-			return NULL;
-		}
-	}
-
-	if(builder->priv->type == MIDGARD_TYPE_USER) {
-
-		if(midgard_user_is_user(user)) {
-
-			if(user->dbpriv->guid == NULL) {
-				
+	if (builder->priv->type == MIDGARD_TYPE_USER) {
+		if (user && midgard_user_is_user(user)) {
+			if (MGD_OBJECT_GUID (user) == NULL) {
 				MIDGARD_ERRNO_SET(builder->priv->mgd, MGD_ERR_ACCESS_DENIED);
 				g_warning("Type incompatible with Midgard Query Builder");
 				return NULL;
 			}
-			
+
 			GValue gval = {0, };
 			g_value_init(&gval, G_TYPE_STRING);
 			g_value_set_string(&gval, user->dbpriv->guid);
 
-			midgard_query_builder_add_constraint(builder, 
-					"guid", "=", &gval);		
-			
+			midgard_query_builder_add_constraint(builder, "guid", "=", &gval);
 		}
-	} 
-        
-        GList *list = midgard_core_qb_set_object_from_query(builder, select_type, NULL);
-        if(list == NULL){
-		if(holder)
-			 holder->elements = 0;
-                return NULL; 
-        }
-        
-       	if(holder)
+	}
+
+	GList *list = midgard_core_qb_set_object_from_query(builder, select_type, NULL);
+	if (list == NULL) {
+		if (holder)
+			holder->elements = 0;
+
+		return NULL;
+	}
+
+	if (holder)
 		holder->elements = g_list_length(list);
 
-	return list;	 
+	return list;
 }
 
+/**
+ * midgard_query_builder_execute:
+ * @builder: #MidgardQueryBuilder instance
+ * @n_objects: a pointer to store number of returned objects
+ * 
+ * Executes the built query.
+ * 
+ * Objects in returned array are #MidgardDBObject derived ones,
+ * and typecasted to base GObject. You can safely typecast them to 
+ * the type, which #MidgardQueryBuilder has been initialized for.
+ * 
+ * In case of any error, #MidgardConnection error is set.
+ * 
+ * Returns: NULL terminated array of objects, or NULL if none found
+ */  
 GObject **midgard_query_builder_execute(
 	MidgardQueryBuilder *builder, guint *n_objects)
 {
@@ -433,7 +627,17 @@ GObject **midgard_query_builder_execute(
 	return (GObject **)objects;	
 }
 
-guint midgard_query_builder_count(MidgardQueryBuilder *builder)
+/** 
+ * midgard_query_builder_count:
+ * @builder: #MidgardQueryBuilder instance
+ *
+ * Returns the number of objects that this query would return when executed
+ * without limit or start offset settings.
+ *
+ * Returns: number of object matched by this query
+ */
+guint 
+midgard_query_builder_count (MidgardQueryBuilder *builder)
 {
 	g_assert(builder != NULL);
 	MIDGARD_ERRNO_SET(builder->priv->mgd, MGD_ERR_OK);
@@ -459,18 +663,6 @@ guint midgard_query_builder_count(MidgardQueryBuilder *builder)
 	return elements;
 }
 
-GList *midgard_query_builder_get_guid(
-		MidgardQueryBuilder *builder)
-{
-	g_assert(builder != NULL);
-	
-	GList *list = 
-		midgard_query_builder_execute_or_count(builder, NULL, MQB_SELECT_GUID);
-	
-	return list;	
-}
-
-
 gchar *midgard_query_builder_get_object_select(
         MidgardQueryBuilder *builder, guint select_type){
 
@@ -491,18 +683,20 @@ gchar *midgard_query_builder_get_object_select(
 	/* We are getting only guids */
 	if(select_type == MQB_SELECT_GUID){
 		
-		g_string_append_printf(select, "COUNT(%s.guid) ", table);
+		g_string_append(select, "COUNT(*) ");
 		return g_string_free(select, FALSE);
 	}
 	
-        /* guid , sitegroup hardcoded ( inherited from MgdObjectClass )
+        /* guid hardcoded ( inherited from MgdObjectClass )
          * metadata properties hardcoded ( defined in midgard_metadata )
          */ 
-	
+	/* FIXME, move this to particular class implementation, so we can reuse MidgardDBObject derived class data only */
+	if (MIDGARD_IS_OBJECT_CLASS (klass))
+			 g_string_append_printf(select, "%s.guid, ", table);
+
 	if (MIDGARD_DBOBJECT_CLASS(klass)->dbpriv->has_metadata) {
 
-		g_string_append_printf(select, 
-				"%s.guid, %s.sitegroup, "
+		g_string_append_printf(select, 	
 				"%s.metadata_creator, "
 				"%s.metadata_created, "
 				"%s.metadata_revisor, "
@@ -529,13 +723,13 @@ gchar *midgard_query_builder_get_object_select(
 				", ", 
 			table, table, table, table, table, table, table, table, table,
 			table, table, table, table, table, table, table, table, table,
-			table, table, table, table, table, table, table); 
-		/* TODO, Set select which suits better */
-	}
-        
-	if(builder->priv->schema->query->select_full != NULL) {
+			table, table, table, table, table); 
+		/* TODO, Set select which suits better */	
+	} 
+
+	if(builder->priv->schema->sql_select_full != NULL) {
 		g_string_append_printf(select, " %s", 
-				builder->priv->schema->query->select_full);
+				builder->priv->schema->sql_select_full);
 	}
 
         return g_string_free(select, FALSE);        
@@ -557,9 +751,9 @@ gchar *midgard_query_builder_get_object_select(
 	midgard_core_metadata_set_##__pname(mdata, &mt); \
 	g_value_unset(&mt); }
 
-static gboolean __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint i)
+static void __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model, gint i)
 {
-	guint col = 1;
+	guint col = 0;
 	const GValue *gvalue;	
 	GValue mt = {0, };
 
@@ -720,7 +914,7 @@ static gboolean __mqb_set_metadata(MidgardMetadata *mdata, GdaDataModel *model,
 			g_value_get_boolean(gvalue);
 	}
 
-	col = 1;
+	return;
 }
 
 GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint select_type, MgdObject *nobject)
@@ -729,14 +923,33 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 
         MgdObject *object;
         guint ret_rows, ret_fields;
+	MidgardConnection *mgd = builder->priv->mgd;
       
 	gchar *sql = midgard_core_qb_get_sql(
 			builder, select_type, 
 			midgard_query_builder_get_object_select(builder, select_type));
 
-	GdaDataModel *model = 
-		midgard_core_query_get_model(
-				builder->priv->mgd, sql);
+	GSList *olist = NULL;
+	MidgardDBObjectClass *dbklass = MIDGARD_DBOBJECT_CLASS(g_type_class_peek(builder->priv->type));
+
+	if (!dbklass) {
+
+		MIDGARD_ERRNO_SET_STRING (mgd, MGD_ERR_INTERNAL, "Failed to peek MidgardDBObjectClass pointer");
+		return NULL;
+	}
+
+	if (dbklass->dbpriv->set_from_sql) {
+
+		if (select_type != MQB_SELECT_GUID) {
+
+			olist = dbklass->dbpriv->set_from_sql(mgd, builder->priv->type, ((const gchar *)sql));
+			g_free(sql);
+
+			return (GList *)olist;
+		}
+	}
+
+	GdaDataModel *model = midgard_core_query_get_model(builder->priv->mgd, sql);
 	g_free(sql);
 
 	if(!model) 
@@ -756,17 +969,30 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 		return list;
 	}
 
-	/* We count guids only */
+	/* We count rows only */
 	if(select_type == MQB_SELECT_GUID) {
 		
 		gvalue = midgard_data_model_get_value_at(model, 0, 0);	
+	
+		if (!gvalue || !G_IS_VALUE (gvalue)) {
+			
+			g_object_unref (model);
+			return 0;
+		}
+
 		MidgardTypeHolder *holder = g_new(MidgardTypeHolder, 1);
-		if (G_IS_VALUE(gvalue) && G_VALUE_HOLDS(gvalue, G_TYPE_INT64))
-			holder->elements = (guint)g_value_get_int64((GValue*)gvalue);
-		else 
-			holder->elements = 0;
+
+		GValue val = {0, };
+		g_value_init (&val, G_TYPE_INT64);
+		g_value_transform (gvalue, &val);
+
+		holder->elements = (guint)g_value_get_int64((GValue*)&val);
+		
 		list = g_list_append(list, holder);	
+		
 		g_object_unref(model);
+		g_value_unset (&val);
+
 		return list;		
 	}
 
@@ -779,19 +1005,28 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 			object = g_object_new(builder->priv->type, NULL);
 
 		MIDGARD_DBOBJECT(object)->dbpriv->mgd = builder->priv->mgd;
-		
-		MidgardDBObjectClass *dbklass = 
-			MIDGARD_DBOBJECT_GET_CLASS(object);
-		
-		if(dbklass->dbpriv->set_from_sql != NULL) {
+				
+		if(dbklass->dbpriv->__set_from_sql != NULL) {
 			
-			dbklass->dbpriv->set_from_sql(MIDGARD_DBOBJECT(object), 
+			dbklass->dbpriv->__set_from_sql(MIDGARD_DBOBJECT(object), 
 					model, rows); 
 
 		} else {
-			
-			/* we have guid, sitegroup and metadata columns first */
-			for (columns = 25; columns < ret_fields; columns++) {	
+		
+			/* Compute number of metadata properties */
+			guint n_md_props = 0;
+
+			if (dbklass->dbpriv->has_metadata) {
+
+				GObjectClass *metadata_class = midgard_object_class_get_metadata_class (MIDGARD_OBJECT_CLASS (dbklass));
+				GParamSpec **pspec = g_object_class_list_properties (metadata_class, &n_md_props);
+				g_free (pspec);
+			}
+
+			guint __cols = n_md_props + 1; /* Add guid column */
+
+			/* we have guid and metadata columns first */
+			for (columns = __cols; columns < ret_fields; columns++) {	
 				
 				gvalue = midgard_data_model_get_value_at(model, columns, rows);
 				const gchar *coltitle =
@@ -803,6 +1038,7 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 
 					guint puint = 0;
 					gint pint = 0;
+					gboolean pbool = 0;
 
 					if (!pspec) {
 						
@@ -823,6 +1059,15 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 							g_object_set(G_OBJECT(object), coltitle, puint, NULL);
 							break;
 
+						case G_TYPE_BOOLEAN:
+							if (G_VALUE_TYPE(gvalue) != G_TYPE_BOOLEAN) {
+								MIDGARD_GET_BOOLEAN_FROM_VALUE(pbool, gvalue);
+								g_object_set(G_OBJECT(object), coltitle, pbool, NULL);
+							} else {
+								g_object_set_property(G_OBJECT(object), coltitle, gvalue);
+							}
+							break;
+
 						default:
 							g_object_set_property(G_OBJECT(object), coltitle, gvalue);
 							break;
@@ -852,33 +1097,27 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 			}	
 		}
 
-		/* Set sitegroup and guid */
+		/* Set guid */
 		const gchar *guid;
-		guint sg = 0;
 		gvalue = midgard_data_model_get_value_at(model, 0, rows);
 		if(G_IS_VALUE(gvalue) && G_VALUE_HOLDS_STRING(gvalue)){
 			guid = g_value_get_string(gvalue);   
 			g_free((gchar *)MIDGARD_DBOBJECT(object)->dbpriv->guid);
 			MIDGARD_DBOBJECT(object)->dbpriv->guid = g_strdup(guid);
 		}
-		gvalue = midgard_data_model_get_value_at(model, 1, rows);
-		if(G_VALUE_HOLDS_UINT(gvalue)) 
-			sg = g_value_get_uint(gvalue);  
-		if(G_VALUE_HOLDS_INT(gvalue))
-			sg = (guint) g_value_get_int(gvalue);
-
-		MIDGARD_DBOBJECT(object)->dbpriv->sg = sg;
 		
-		/* Set metadata */   
-		GParamSpec *pspec = 
-			g_object_class_find_property(G_OBJECT_GET_CLASS(object), "metadata");
+		/* Set metadata */
+	     	if (dbklass->dbpriv->has_metadata) {
+	
+			GParamSpec *pspec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), "metadata");
 
-		if(pspec){
-
-			MidgardMetadata *mdata = object->metadata;	
-			if(mdata)
-				__mqb_set_metadata(mdata, model, rows);
-		}
+			if(pspec){
+	
+				MidgardMetadata *mdata = object->metadata;	
+				if(mdata)
+					__mqb_set_metadata(mdata, model, rows);
+			}
+		}	
 
 		list = g_list_append(list, G_OBJECT(object));                		
 	}
@@ -886,28 +1125,37 @@ GList *midgard_core_qb_set_object_from_query(MidgardQueryBuilder *builder, guint
 	g_object_unref(model);
 	return list;	
 }
-
-gboolean midgard_query_builder_join(
-		MidgardQueryBuilder *builder, const gchar *prop, 
-		const gchar *jobject, const gchar *jprop)
-{
-	g_assert(prop != NULL);
-	g_assert(jobject != NULL);
-	g_assert(jprop != NULL);
-
-	g_warning("FIXME");
-}
-
  
-/* Returns type name of the type which is currently used by Query Builder. */
-const gchar *midgard_query_builder_get_type_name(
-		MidgardQueryBuilder *builder)
+/**
+ * midgard_query_builder_get_type_name:
+ * @builder: #MidgardQueryBuilder instance
+ * 
+ * Returns type name of the type which is currently used by Query Builder.
+ * 
+ * This function should be used on language binding level , when internal
+ * Query Builder's instance is already created and language binding object 
+ * should be instanciated.
+ *
+ * Returned type name is a pointer and is owned by GLib system.
+ * Caller shouldn't free it.
+ *
+ * Returns: name of the class, which query builder is initialized for.
+ */ 
+const gchar*
+midgard_query_builder_get_type_name (MidgardQueryBuilder *builder)
 {
 	g_assert(builder != NULL);
 	return g_type_name(builder->priv->type);
 }
 
-void midgard_query_builder_include_deleted(MidgardQueryBuilder *builder)
+/**
+ * midgard_query_builder_include_deleted:
+ * @builder: #MidgardQueryBuilder instance
+ * 
+ * Query all objects, deleted and undeleted. 
+ */ 
+void 
+midgard_query_builder_include_deleted (MidgardQueryBuilder *builder)
 {
 	g_assert(builder);
 
@@ -929,8 +1177,6 @@ static void _midgard_query_builder_class_init(
 	MidgardQueryBuilderClass *klass = MIDGARD_QUERY_BUILDER_CLASS (g_class);
 	
 	gobject_class->finalize = _midgard_query_builder_finalize;
-	klass->set_lang = midgard_query_builder_set_lang;
-	klass->unset_languages = midgard_query_builder_unset_languages;
 	klass->add_constraint = midgard_query_builder_add_constraint;
 	klass->add_constraint_with_property = 
 		midgard_query_builder_add_constraint_with_property;
