@@ -272,9 +272,12 @@ static void __transform_value(MidgardQueryConstraint *self)
 
 		} else {
 
-			g_warning("Invalid value conversion from %s to %s",
-					G_VALUE_TYPE_NAME(value),
-					G_VALUE_TYPE_NAME(target));
+			if (value && target) {
+	
+				g_warning("Invalid value conversion from %s to %s",
+						G_VALUE_TYPE_NAME(value),
+						G_VALUE_TYPE_NAME(target));
+			}
 		}
 	}
 
@@ -377,23 +380,6 @@ gboolean midgard_query_constraint_build_condition(
 			return FALSE;
 		}
 		
-		/* This comparison is done to avoid ugly endless loops for 
-		 * properties which are not parent or up ones 
-		 * This should be changed when midgard_tree is implemented */
-		const gchar *parent_field, *up_field = NULL;
-		
-		parent_field =
-			midgard_object_class_get_property_parent(
-					(MidgardObjectClass *)constraint->priv->klass);
-		if(parent_field == NULL)
-			parent_field = "";
-		
-		up_field =
-			midgard_object_class_get_property_up(					
-					(MidgardObjectClass *)constraint->priv->klass);
-		if(up_field == NULL)
-			up_field = "";
-
 		g_string_append(cond, "IN ");
 	
 		guint tid = 0;
@@ -455,6 +441,24 @@ gboolean midgard_query_constraint_build_condition(
 	return FALSE;
 }
 
+void            
+midgard_query_constraint_set_builder (MidgardQueryConstraint *self, MidgardQueryBuilder *builder)
+{
+	g_assert(self != NULL);
+	g_assert(builder != NULL);
+
+	self->priv->builder = builder;
+}
+
+void            
+midgard_query_constraint_set_class (MidgardQueryConstraint *self, MidgardDBObjectClass *klass)
+{
+	g_assert(self != NULL);
+	g_assert(klass != NULL);
+
+	self->priv->klass = G_OBJECT_CLASS(klass);
+}
+
 /* GOBJECT ROUTINES */
 
 MidgardQueryConstraintPrivate *midgard_query_constraint_private_new(void)
